#
# $Header:   //ARCHIVE/PVCSROOT/Archive_NLS/Scripts/Upload_CreateAuditMsg.ksv   1.3.1.1   Aug 24 2001 14:22:08   Forthman  $ 
#

bUploadOK=1;

usage	()
{
  print ''
	print 'Usage: '$0' -c<cid> -i<id> -f<audit_msg_file> -o<comm_output_file> [-e]'
  print ''
  print '  Required Param(s):'
  print '  -c<cid>                 Object''s ClassID'
  print '  -i<id>                  Object''s ID'
  print '  -f<audit_msg_file>      Upload Audit Message File'
  print '  -e<comm_output_file>    Communications output file'
  print ''
  print '  Optional Param(s):'
  print '  -e                      Upload communications FAILED'
  print ''
  print '  Sample:'
  print '    '$0' -iC:/TEMP/MVSVariableRegistration.19990729130714_499.JCL.H7X0.FTP.AUDIT.MSG'
}

# Process input parameters
while getopts "c:i:f:o:e?" opt; do
	case $opt in
	c)  cid=$OPTARG;;
	i)  id=$OPTARG;;
	f)  audit_msg_file=$OPTARG;;
	o)  comm_output_file=$OPTARG;;
	e)  bUploadOK=0;;
	\?) usage
		  exit 16
		  ;;
	esac
done
shift $(($OPTIND - 1))

# Check for required params
typeset -i bMissingParam=0;
if [ "$cid" = "" ];          then echo 'ERROR: Missing required param -c<cid>'; bMissingParam=1; fi
if [ "$id" = "" ];           then echo 'ERROR: Missing required param -i<id>'; bMissingParam=1; fi
if [ "$audit_msg_file" = "" ];   then echo 'ERROR: Missing required param -f<audit_msg_file>'; bMissingParam=1; fi
if [ "$comm_output_file" = "" ];   then echo 'ERROR: Missing required param -o<comm_output_file>'; bMissingParam=1; fi
if [ $bMissingParam -eq 1 ]; then
	usage
	exit 17
fi

create_msg()
{
  echo 'Connecting to: '$db_server
  echo '     Username: '$db_user
  echo '     Password: '$db_password:

  echo 'cid:           '$obj_cid
  echo 'id:            '$obj_id
  print -r 'MessageID:     '"$msgID"
  print -r 'Description:   '"$msgDescr"
  if [ $bUploadOK -eq 0 ]; then;
    echo "ERROR: Communications error during upload"
  fi

  # if not an MJOB (Mainframe Job), send the audit msg to the OS
  if [ ! $obj_cid = BATC -a ! $obj_cid = CICS -a ! $obj_cid = STC -a ! $obj_cid = IMS -a ! $obj_cid = IDMS -a ! $obj_cid = OMEG -a ! $obj_cid = DB2 -a ! $obj_cid = D2SS ]; then;
    obj_cid=$cid
    obj_id=$id
  fi

  jcl_job=""
  echo $comm_output_file | grep -q ERROR
  if [ $? -ne 0 ]; then
    ftp_results_file="${comm_output_file%.TXT}".FTP.RESULTS.FTP
    if [ -f "$ftp_results_file" ]; then
      jcl_job=`awk '/It is known to JES as/ {print $7}' "$ftp_results_file"`
    fi
  fi;

  # create output filename(s)
  isql_input_file=$output_path'/Upload_CreateAuditMsg.'$timestamp'_'$$.ISQL.IN.TXT
  isql_output_file=$output_path'/Upload_CreateAuditMsg.'$timestamp'_'$$.ISQL.OUT.TXT

  # create the SQL input
  print -r "
    use Object
    go
    set nocount on
    go
    declare @now datetime
    declare @bUploadOK tinyint
    declare @reason_cd varchar(255)
    declare @jcljob varchar(255)

    select @now=getdate(), @bUploadOK=$bUploadOK, @jcljob='$jcl_job'
    if @bUploadOK=1
    begin
      -- may be send to OS or to specific object (obj_cid, obj_id)
      select @reason_cd='Sent ''$msgDescr'' message to Source/390.'
      if @jcljob<>''
        select @reason_cd=@reason_cd+'  JES JobID='''+@jcljob+'''.'
      select @reason_cd=@reason_cd+'  See ''$comm_output_file'' on database host for more information.'
      exec asisp_createMESG '', $obj_cid, $obj_id, @TIME=@now, @MESSAGE='$msgID', @REASON_CD=@reason_cd
    end
    else
      -- always send FAILURE to the OS (cid, id)
      -- OR
      -- may be send to OS or to specific object (obj_cid, obj_id)
      exec asisp_createMESG '', $obj_cid, $obj_id, @TIME=@now, @MESSAGE='ASIUPLOAD_FAIL',
                          @REASON_CD='FAILED to send ''$msgDescr'' message to Source/390.  See ''$comm_output_file'' on database host for more information about communications failure.'
		" > "$isql_input_file"

  # Execute SQL in a loop while deadlocking
  error_code=9999;
  last_error_code=0;
  while [ $error_code -ne 0 ]; do;

    # make the SQL call
    isql -S$db_server -U$db_user -P$db_password -w5000 -t120 -l120 -n -i"$isql_input_file" -o"$isql_output_file"

    # evaluate the results
    error_code=0; # reset before test
    eval $(grep "Msg.*Level.*State" "$isql_output_file" | awk '{split($2,v,/,/); printf("error_code=%d\n",v[1])}')

    # if error has gone away, log the success
    if [ $error_code -eq 0 -a $last_error_code -ne 0 ]; then
      echo $(date) >> "$isql_output_file".ERROR
      echo 'OK - SQL call succeed.' >> "$isql_output_file".ERROR
    fi;
    last_error_code=$error_code

    # process errors (if any)
    if [ $error_code -ne 0 ]; then;
      echo SQL Error: $error_code;
      echo $(date) >> "$isql_output_file".ERROR
      cat "$isql_output_file" >> "$isql_output_file".ERROR

      # break if not deadlock: 1205
      if [ $error_code -ne 1205 ]; then;
        break;
      fi;
      sleep 10;
    fi
  done;

  # cleanup
  if [ -f "$isql_input_file" ]; then;
    rm "$isql_input_file";
  fi
  if [ -f "$isql_output_file" ]; then;
    rm "$isql_output_file";
  fi
}

main()
{
  timestamp=`date +"%Y%m%d%H%M%S"`

  # Get Database Server, User, Password from Registry
  eval `regdmp "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\DB" | awk '/Server/ {printf("db_server=%s\n",$3);} /User/ {printf("db_user=%s\n",$3);} /Password/ {printf("db_password=%s\n",$3);}'`

  # determine the output directory
  eval `regdmp "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\Log" | awk '/Directory/ {gsub(/\\\\/, "/", $3); printf("output_path=%s\n",$3);}'`
  if [ "$output_path" = "" ]; then;
    if [ "$TEMP" != "" ]; then; output_path=`echo $TEMP | awk '{gsub(/\\\\/, "/"); print;}'`;
    elif [ "$TMP" != "" ]; then; output_path=`echo $TMP | awk '{gsub(/\\\\/, "/"); print;}'`;
    elif [ "$TMPDIR" != "" ]; then; output_path=`echo $TMPDIR | awk '{gsub(/\\\\/, "/"); print;}'`;
    fi;
  fi

  IFS='|';
  while read -r obj_cid obj_id msgID msgDescr;
  do 
    create_msg
  done < "$audit_msg_file";

#  rm "$audit_msg_file"
}

main
