-- $Header:   //ARCHIVE/Archive_NLS/DB/SQL/CreateAllocProcs.sqv   1.0   Aug 14 2001 08:28:00   BRENT  $
--
-- CreateAllocProcs.sql
--
-- Module Description
--   Creates the delete trigger for any link table that is missing it

USE Object
GO

---------------------------------------------------------------------------------------------------
-- CreateAllocProcs
PRINT 'Creating procedure CreateAllocProcs'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'CreateAllocProcs' AND type = 'P')
  DROP PROCEDURE CreateAllocProcs
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/SQL/CreateAllocProcs.sqv  $
-- $Revision:   1.0  $
-- $Date:   Aug 14 2001 08:28:00  $
-- $Author:   BRENT  $
-- Creates the alloc_<CID> procedures for all currently defined classes
CREATE PROCEDURE CreateAllocProcs
AS
  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT

  SET NOCOUNT ON
  DECLARE procc INSENSITIVE CURSOR FOR
    SELECT RTRIM(oc.cid), so.name
    FROM sysobjects AS so
      JOIN obj_class AS oc ON so.name = 'alloc_' + RTRIM(oc.cid)
    WHERE
      so.type = 'P'
    ORDER BY
      so.name
  OPEN procc
  DECLARE @cid NVARCHAR(4), @proc sysname, @sql VARCHAR(8000)
  FETCH NEXT FROM procc INTO @cid, @proc
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SET @sql = "-- $Archive:   //ARCHIVE/Archive_NLS/DB/SQL/CreateAllocProcs.sqv  $
-- $Revision:   1.0  $
-- $Date:   Aug 14 2001 08:28:00  $
-- $Author:   BRENT  $
-- Allocation procedure for class " + @cid + "
CREATE PROCEDURE alloc_" + @cid + "
  @count    INT=1
AS
  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION alloc_" + @cid + "
  ELSE
    SAVE TRANSACTION alloc_" + @cid + "

  DECLARE @last_id INT
  SELECT @last_id=last_id+1 FROM " + @cid + "_ID WITH (TABLOCKX)

  UPDATE " + @cid + "_ID SET last_id=last_id+@count 
  
  IF @tcount = 0
    COMMIT TRANSACTION alloc_" + @cid + "

  RETURN @last_id
"
    RAISERROR('Creating procedure %s', 0, 1, @proc) WITH NOWAIT
    IF @tcount = 0
      BEGIN TRANSACTION CreateAllocProcs
    ELSE
      SAVE TRANSACTION CreateAllocProcs

    EXEC("
    DROP PROCEDURE " + @proc
    )
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRANSACTION CreateAllocProcs
      GOTO NextProc
    END

    EXEC(@sql)
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRANSACTION CreateAllocProcs
      GOTO NextProc
    END

    IF @tcount = 0
      COMMIT TRANSACTION CreateAllocProcs
NextProc:
    FETCH NEXT FROM procc INTO @cid, @proc
  END
  DEALLOCATE procc
GO

EXEC CreateAllocProcs
GO
