-- $Header:   //ARCHIVE/Archive_NLS/db/sql/Process_GRBT_Event.sqv   1.1   Aug 12 2001 19:56:00   BRENT  $
--
-- Process_GRBT_Event.sql
--
-- Module Description
--   Defines the Process_GRBT_Event stord procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- Process_GRBT_Event
PRINT 'Creating procedure Process_GRBT_Event'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'Process_GRBT_Event' AND type = 'P')
  DROP PROCEDURE Process_GRBT_Event
GO

-- $Archive:   //ARCHIVE/Archive_NLS/db/sql/Process_GRBT_Event.sqv  $
-- $Revision:   1.1  $
-- $Date:   Aug 12 2001 19:56:00  $
-- $Author:   BRENT  $
-- Processes an event received on GRBE transaction
CREATE PROCEDURE Process_GRBT_Event
  @MESSAGE      NVARCHAR(255)=NULL,
  @REASON_CD    NVARCHAR(2000)=NULL,
  @DATE_TIME    DATETIME=NULL,
  @SEQ_NO       NVARCHAR(100)=NULL,
  @SourceCID    ClassID=NULL,
  @SourceID     ObjID=NULL,
  @GRBEName     ObjName=NULL,
  @EventCID     ClassID=NULL,
  @verbose      TINYINT=0
AS
  SET NOCOUNT ON

  -- Standard transaction logic
  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION Process_GRBT_Event
  ELSE
    SAVE TRANSACTION Process_GRBT_Event

  -- Find the specified GRBE Region
  DECLARE @grbe_id ObjID
  SELECT @grbe_id = grbe.id
  FROM GRBERegion_V AS grbe JOIN containment AS c ON c.dst_cid = 'GRBE' AND c.dst_id = grbe.id
  WHERE
    grbe.name = @GRBEName
    AND c.src_cid = @SourceCID
    AND c.src_id = @SourceID
    AND c.deleted = 0
  -- If not found then create it
  IF @grbe_id IS NULL
  BEGIN
    EXEC @grbe_id = asisp_createGRBE @GRBEName, @SourceCID, @SourceID
    IF @@ERROR <> 0 OR @grbe_id < 0
    BEGIN
      RAISERROR('Process_GRBT_Event: Could not create GRBE Region [%s] under %s-%d', 3, 1, @GRBEName, @SourceCID, @SourceID)
      ROLLBACK TRANSACTION Process_GRBT_Event
      RETURN -30000
    END
  END
  ELSE
    -- Update the mtime column on the object to indicate it has received an event
    UPDATE GRBERegion_V SET mtime = CURRENT_TIMESTAMP WHERE id = @grbe_id

  DECLARE @handle INT, @hr INT, @errtext VARCHAR(255), @rc INT
  IF @handle IS NULL  -- Create the regular expression handler
  BEGIN
    EXEC @hr = sp_OACreate 'TMRegExp.RegExContext', @handle OUT
    IF @hr <> 0
    BEGIN
      EXEC sp_OAGetErrorInfo NULL, NULL, @errtext OUT
      RAISERROR('Process_GRBT_Event: %s', 3, 1, @errtext)
      SET @rc = -30000
      GOTO RegexError
    END
  END

  -- Set the regular expression based on whether @REASON_CD contains 'INATIVIDA' (Inactive) or 'ATIVIDA' (Active)
  DECLARE @regexpr VARCHAR(255)
  IF @REASON_CD LIKE '%INATIVADA%'
    SET @regexpr = '\([^ ][^ ][^ ]\)\([^ ]*\)[ ][ ]*\(INATIVADA\)\( MCMD083C.A \)\?'
  ELSE
    SET @regexpr = '\([^ ][^ ][^ ]\)\([^ ]*\)[ ][ ]*\(ATIVADA\)\( MCMD082C.A \)\?'
  -- Compile this expression
  EXEC @hr = sp_OAMethod @handle, 'Compile', NULL, @regexpr, 0
  IF @hr <> 0
  BEGIN
    EXEC sp_OAGetErrorInfo @handle, NULL, @errtext OUT
    RAISERROR('Process_GRBT_Event: %s', 3, 1, @errtext)
    SET @rc = -30000
    GOTO RegexError
  END

  -- Strip leading '+' from message id if present
  IF SUBSTRING(@MESSAGE, 1, 1) = '+'
    SET @MESSAGE = SUBSTRING(@MESSAGE, 2, LEN(@MESSAGE)-1)

  CREATE TABLE #tran 
  (
    name      NVARCHAR(8) NOT NULL,
    MESSAGE   NVARCHAR(255) NULL,
    REASON_CD NVARCHAR(255) NOT NULL,
    id        ObjID NULL
  )
  
  WHILE LEN(@REASON_CD) > 1
  BEGIN
    RAISERROR('Top of loop. REASON_CD=[%s]', 0, 1, @REASON_CD) WITH NOWAIT
    -- Search the text
    EXEC @hr = sp_OAMethod @handle, 'Exec', NULL, @REASON_CD, 0
    IF @hr <> 0
    BEGIN
      EXEC sp_OAGetErrorInfo @handle, NULL, @errtext OUT
      IF @hr <> 0x800A0801 -- If not a 'no match' error then this is treated as a processing error
      BEGIN
        RAISERROR('Process_GRBT_Event: %s', 3, 1, @errtext)
        SET @rc = -30000
        GOTO RegexError
      END
    END

    -- See if there were matches
    DECLARE @count INT
    SET @count = 0
    EXEC @hr = sp_OAGetProperty @handle, 'Count', @count OUTPUT
    IF @hr <> 0
    BEGIN
      EXEC sp_OAGetErrorInfo @handle, NULL, @errtext OUT
      RAISERROR('Process_GRBT_Event: %s', 3, 1, @errtext)
      SET @rc = -30000
      GOTO RegexError
    END

    IF @count >= 4
    BEGIN
      DECLARE @match VARCHAR(255), @name VARCHAR(8), @step VARCHAR(8), @state VARCHAR(16)
      EXEC @hr = sp_OAMethod @handle, 'Match', @match OUTPUT, 0
      IF @hr <> 0
      BEGIN
        RAISERROR('Process_GRBT_Event: %s', 3, 1, @errtext)
        SET @rc = -30000
        GOTO RegexError
      END
      EXEC @hr = sp_OAMethod @handle, 'Match', @name OUTPUT, 1
      IF @hr <> 0
      BEGIN
        RAISERROR('Process_GRBT_Event: %s', 3, 1, @errtext)
        SET @rc = -30000
        GOTO RegexError
      END
      EXEC @hr = sp_OAMethod @handle, 'Match', @step OUTPUT, 2
      IF @hr <> 0
      BEGIN
        RAISERROR('Process_GRBT_Event: %s', 3, 1, @errtext)
        SET @rc = -30000
        GOTO RegexError
      END
      EXEC @hr = sp_OAMethod @handle, 'Match', @state OUTPUT, 3
      IF @hr <> 0
      BEGIN
        RAISERROR('Process_GRBT_Event: %s', 3, 1, @errtext)
        SET @rc = -30000
        GOTO RegexError
      END

      RAISERROR('match = [%s]', 0, 1, @match) WITH NOWAIT
      RAISERROR('name  = [%s]', 0, 1, @name ) WITH NOWAIT
      RAISERROR('step  = [%s]', 0, 1, @step ) WITH NOWAIT
      RAISERROR('state = [%s]', 0, 1, @state) WITH NOWAIT

      -- Make sure the name has been parsed. If not, we don't try to recover since we have no idea what's wrong
      IF @name IS NULL
        GOTO EndOfLoop

      INSERT INTO #tran
      VALUES
      ( 
        @name,
        @MESSAGE + ' ' + @step, 
        CASE @MESSAGE WHEN 'MCMD083C.A' THEN @MESSAGE + ' ' + @name + @step + ' ' + @state ELSE 'OK' END,
        NULL
      )

      -- If there was a trailing message id then this applies to the next transaction
      IF @count = 5
      BEGIN
        DECLARE @msgid VARCHAR(255)
        EXEC @hr = sp_OAMethod @handle, 'Match', @msgid OUTPUT, 4
        IF @hr <> 0
        BEGIN
          RAISERROR('Process_GRBT_Event: %s', 3, 1, @errtext)
          SET @rc = -30000
          GOTO RegexError
        END
        SET @MESSAGE = RTRIM(LTRIM(@msgid))
        RAISERROR('msgid = [%s]', 0, 1, @msgid) WITH NOWAIT
      END

      IF @verbose = 1
      BEGIN
        PRINT 'Mat: [' + @match + ']'
        PRINT 'OLD: [' + @REASON_CD + ']'
      END

      SET @REASON_CD = SUBSTRING(@REASON_CD, LEN(@match)+2, LEN(@REASON_CD)-LEN(@match)+1)

      IF @verbose = 1
      BEGIN
        PRINT 'NEW: [' + @REASON_CD + ']'
      END
    END
  END
EndOfLoop:

  IF EXISTS (SELECT * FROM #tran)
  BEGIN
    UPDATE t SET id = grbt.id
    FROM
      #tran AS t
      JOIN GRBETransaction_V AS grbt ON grbt.name = t.name
      JOIN containment AS c ON c.dst_cid = 'GRBT' AND c.dst_id = grbt.id
    WHERE
      c.deleted = 0
      AND c.src_id = @grbe_id

    -- Create any transactions required
    DECLARE @nNew INT
    SELECT @nNew = COUNT(DISTINCT name) FROM #tran WHERE id IS NULL
    IF @nNew > 0
    BEGIN
      CREATE TABLE #new ( name NVARCHAR(8) NOT NULL, rowno INT IDENTITY(0,1) )
      INSERT INTO #new SELECT DISTINCT name FROM #tran WHERE id IS NULL
      DECLARE @base_id ObjID
      EXEC @base_id = alloc_GRBT @nNew
      INSERT INTO GRBETransaction_C(id, name, _ChildEventMatrixID, _ExceptionMatrixID)
        SELECT @base_id+rowno, name, 0, 0
        FROM #new
      INSERT INTO GRBE_GRBT_GRTL(src_id, dst_id)
        SELECT @grbe_id, @base_id+rowno
        FROM #new
      UPDATE t SET id = @base_id+n.rowno FROM #tran AS t JOIN #new AS n ON n.name = t.name
    END

    -- Send event to object
    IF @EventCID = 'EXCP'
      INSERT INTO StagedEXCP(parent_cid, parent_id, TIME, SEQ_NO, OBJECT_NAME, EXCP_NAME, EXCP_CD, nEXCH)
        SELECT 'GRBT', t.id, @DATE_TIME, @SEQ_NO, t.name, t.MESSAGE, t.REASON_CD, 1
        FROM #tran AS t
    ELSE IF @EventCID = 'MESG'
      INSERT INTO StagedMESG(parent_cid, parent_id, TIME, SEQ_NO, OBJECT_NAME, NATIVE_KEY, STATE, MESSAGE, REASON_CD, DISCOVERY_IND)
        SELECT 'GRBT', t.id, @DATE_TIME, @SEQ_NO, t.name, '', '', t.MESSAGE, t.REASON_CD, 'MON'
        FROM #tran AS t
    ELSE
    BEGIN
      DECLARE @DATE_TIME_STR NVARCHAR(40)
      SET @DATE_TIME_STR = CONVERT(NVARCHAR(40), @DATE_TIME, 121)
      RAISERROR('Process_GRBT_Event: Discarding the following event due to no EventCID mapping:', 2, 1)
      RAISERROR('  MESSAGE    = [%s]', 2, 1, @MESSAGE)
      RAISERROR('  REASON_CD  = [%s]', 2, 1, @REASON_CD)
      RAISERROR('  SEQ_NO     = [%s]', 2, 1, @SEQ_NO)
      RAISERROR('  DATE_TIME  = [%s]', 2, 1, @DATE_TIME_STR)
    END
  END

  -- Standard transaction logic
  IF @tcount = 0
    COMMIT TRANSACTION Process_GRBT_Event
  RETURN 0
RegexError:
  RETURN @rc
GO
