-- $Header$
--
-- _GetDependencies.sql
--
-- Module Description
--   Defines the _GetDependencies stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- _GetDependencies
PRINT 'Creating procedure _GetDependencies'
GO

IF EXISTS(SELECT * FROM sysobjects WHERE name = '_GetDependencies' AND type = 'P')
  DROP PROC _GetDependencies
GO

-- $Archive:   //ARCHIVE/Archive_NLS/db/sql/_GetDependencies.sql  $
-- $Revision:   1.0  $
-- $Author:   Marc  $
-- $Date:   Jun 29 2001 11:37:08  $
-- Procedure used by 'Business Impact' function to return an inverted hierarchy
-- where calling object is on the top, and all objects that are linked to it from
-- source links are shown underneath it.
CREATE PROCEDURE _GetDependencies
  @cid ClassID,
  @id ObjID,
  @securityLevel INT = 0
AS
  SET NOCOUNT ON


  -- Temp table holding all generations of objects being collected
  CREATE TABLE #dependencies
  (
    src_cid ClassID,
    src_id  ObjID,
    dst_cid ClassID,

    dst_id  ObjID,
    phy_cid ClassID,
    phy_id  ObjID,
    gen     INT,
    node    NVARCHAR(255)
  )

  -- Temp table for holding the objects collected in one generation. Needed to generate the IDENTITY
  -- values for the node ids.
  CREATE TABLE #gen
  (
    src_cid ClassID,
    src_id  ObjID,
    dst_cid ClassID,
    dst_id  ObjID,
    gen     INT,
    nodeid  INT IDENTITY(1,1)
  )

  -- Temp table for holding the objects collected in one generation. Needed to generate the IDENTITY
  -- values for the node ids.
  CREATE TABLE #gen1
  (
    src_cid ClassID,
    src_id  ObjID,
    dst_cid ClassID,
    dst_id  ObjID,
    gen     INT,
    nodeid  INT IDENTITY(1,1),
    link_type ClassID
  )

  -- If the calling object is an LOB then we resolve the LOB to the physical object before starting
  DECLARE @phy_obj_cid ClassID, @phy_obj_id ObjID
  IF @cid = 'LOB'
   BEGIN
    EXEC asisp_followphysicallink @id, @cid out, @id out
    SELECT @phy_obj_cid=@cid
    SELECT @phy_obj_id=@id
   END

  -- Seed table with row for object and its immediate links
  DECLARE @gen INT, @rc INT, @rc2 INT, @del_flag INT, @ins_count INT
  SELECT @gen = 0, @rc = 1, @del_flag=0, @ins_count=0
  INSERT #dependencies SELECT @cid, @id, '', 0, @cid, @id, @gen, '00001'
  SELECT @gen = @gen + 1

  -- if the user does not have permission to view the physical tree, then skip the physical objects
  IF (@securityLevel IS NOT NULL AND @securityLevel <> 0) OR @cid = 'LOB'
  BEGIN
    INSERT #gen SELECT src_cid, src_id, dst_cid, dst_id, @gen FROM containment c where dst_cid = @cid AND dst_id = @id AND deleted = 0
      AND src_cid NOT IN ('LOBC', 'BUSC')
    SELECT @rc = @@ROWCOUNT

  -- if no row in containment
    IF @rc=0
    BEGIN
	DECLARE @objname NVARCHAR(24), @tname NVARCHAR(26), @rc_1 INT, @sql NVARCHAR(2000)
	DECLARE @childcid ClassID, @childid ObjID
	DECLARE @parentcid ClassID, @parentid ObjID
	DECLARE @findsysname sysname
	SELECT @rc_1=0
	SELECT @childcid=@cid, @childid=@id
	WHILE @rc_1=0
	  BEGIN
		SELECT @objname=cname FROM obj_class WHERE cid=@childcid
		SELECT @tname=@objname+"_V"
		SELECT @findsysname=name FROM sysobjects WHERE name=@tname
		IF @findsysname=NULL
			SELECT @rc_1=-1
		ELSE
		BEGIN		
		-- find parentcid & parentid
		  SELECT @sql="DECLARE findpcid cursor for SELECT _ParentCID, _ParentID FROM " + @objname + "_V WHERE id=" + CAST(@childid AS NVARCHAR(100))
		  EXEC(@sql)
		  OPEN findpcid
		  FETCH NEXT FROM findpcid INTO @parentcid, @parentid
		  DEALLOCATE findpcid

		  INSERT INTO containment VALUES (@childcid, @childid, @parentcid, @parentid, 0)
		  SELECT @childcid=@parentcid, @childid=@parentid

		  DECLARE @midcid ClassID
		  SELECT @midcid=dst_cid FROM containment WHERE dst_cid=@parentcid AND dst_id=@parentid AND deleted = 0
		  IF @midcid=NULL
			SELECT @rc_1 = 0
		  ELSE
			SELECT @rc_1 = 1

		  SELECT @ins_count = @ins_count + 1
		END
	  END

    IF @rc_1 <> -1
    BEGIN
      INSERT #gen SELECT src_cid, src_id, dst_cid, dst_id, @gen FROM containment c where dst_cid = @cid AND dst_id = @id AND deleted = 0
        AND src_cid NOT IN ('LOBC', 'BUSC')
      SELECT @del_flag=1
      SELECT @rc = @@ROWCOUNT
    END -- if rc_1 <> -1
    END -- if rc=0 

  END

  -- Seed LOB links. We skip over the immediately linked LOBs in order to avoid the awkwardness of displaying the a root object
  -- surrounded by LOB objects that look just like it.
  INSERT #gen 
    SELECT l2.src_cid, l2.src_id, l2.dst_cid, l2.dst_id, @gen 
    FROM link l1, link l2
    WHERE 
      l1.link_type = 'LOBL' AND l1.dst_cid = @cid AND l1.dst_id = @id AND l1.deleted = 0
      AND l2.link_type LIKE 'LOB[CL]' AND l2.dst_cid = l1.src_cid AND l2.dst_id = l1.src_id AND l2.deleted = 0
    AND l2.src_cid NOT IN ('LOBC', 'BUSC')
  SELECT @rc = @rc + @@ROWCOUNT

  INSERT #dependencies
    SELECT g.src_cid, g.src_id, g.dst_cid, g.dst_id, g.src_cid, g.src_id, g.gen, 
      '00001.' + REPLICATE('0', 5-LEN(CONVERT(NVARCHAR(9), nodeid))) + CONVERT(NVARCHAR(9), nodeid)
    FROM #gen g

  -- Now traverse up the physical and logical trees
  WHILE @rc > 0 AND @gen < 10
  BEGIN
--SELECT @gen AS GEN
    -- Get the containing objects for the previous generation
    SELECT @gen = @gen + 1
    TRUNCATE TABLE #gen
    -- Follow containment links

    INSERT #gen
      SELECT DISTINCT c.src_cid, c.src_id, c.dst_cid, c.dst_id, @gen
      FROM containment c, #dependencies prev
      WHERE prev.gen = @gen - 1 AND c.dst_cid = prev.src_cid AND c.dst_id = prev.src_id AND c.src_cid NOT IN ('BUSC', 'LOBC') AND c.deleted = 0
    SELECT @rc = @@ROWCOUNT

--SELECT 'containment' as 'containment', @gen AS CurrentGen, * FROM #gen

    -- Follow logical links for LOBs
    INSERT #gen
      SELECT DISTINCT l.src_cid, l.src_id, l.dst_cid, l.dst_id, @gen
      FROM link l, #dependencies prev
      WHERE prev.gen = @gen - 1 AND prev.src_cid='LOB' AND l.link_type = 'LOBL'
        AND l.dst_cid = prev.src_cid AND l.dst_id = prev.src_id AND l.deleted = 0 AND l.src_cid <> 'LOB'
    SELECT @rc = @rc + @@ROWCOUNT

--SELECT 'LOBL' as 'LOBL', @gen AS CurrentGen, * FROM #gen

    TRUNCATE TABLE #gen1
    INSERT #gen1
      SELECT l.src_cid, l.src_id, l.dst_cid, l.dst_id, @gen, l.link_type
      FROM link l, #dependencies prev
      WHERE prev.gen = @gen - 1 AND prev.src_cid='LOB' AND l.link_type = 'LOBL'
        AND l.dst_cid = prev.src_cid AND l.dst_id = prev.src_id AND l.src_cid = 'LOB' AND l.deleted = 0
    SELECT @rc2 = @@ROWCOUNT
    SELECT @rc = @rc + @rc2
    WHILE @rc2 > 0
    BEGIN
      INSERT #gen1
        SELECT l.src_cid, l.src_id, l.dst_cid, l.dst_id, @gen, l.link_type
        FROM link l, #gen1 prev
        WHERE prev.src_cid = 'LOB' AND l.link_type = 'LOBL'
          AND l.dst_cid = prev.src_cid AND l.dst_id = prev.src_id AND l.deleted = 0
      SELECT @rc2 = @@ROWCOUNT
      DELETE FROM #gen1 WHERE src_cid = dst_cid AND src_id = dst_id
    END

    -- Resolve LOBL links FROM LOB to LOB into LOBC links
    INSERT INTO #gen1 SELECT l.src_cid, l.src_id, prev.dst_cid, prev.dst_id, @gen, l.link_type
      FROM #gen1 prev, link l
      WHERE prev.src_cid = 'LOB' AND l.link_type = 'LOBC' AND prev.link_type = 'LOBL' AND gen = @gen
        AND l.dst_cid = prev.src_cid AND l.dst_id = prev.src_id AND l.deleted = 0
        INSERT #gen SELECT src_cid, src_id, dst_cid, dst_id, gen
      FROM #gen1 WHERE @gen = gen AND link_type <> 'LOBL'

    INSERT #dependencies
      SELECT g.src_cid, g.src_id, g.dst_cid, g.dst_id, g.src_cid, g.src_id, g.gen,
        prev.node + '.' + REPLICATE('0', 5-LEN(CONVERT(NVARCHAR(9), nodeid))) + CONVERT(NVARCHAR(9), nodeid)
      FROM #dependencies prev, #gen g
      WHERE g.dst_cid = prev.src_cid AND g.dst_id = prev.src_id

  END

--SELECT 'AFTER' AS 'AFTER', * FROM #dependencies

  -- Now we need to resolve all LOBs to their linked physical objects (if they have any). This drives the display of the object type
  UPDATE d SET phy_cid = ll.phy_cid, phy_id = ll.phy_id 
    FROM #dependencies AS d JOIN lob_link AS ll ON d.phy_cid = 'LOB' AND d.phy_id = ll.dst_id
  WHERE
    ll.phy_cid IS NOT NULL

  -- Create the temp table used to display results to the caller. We could collapse this and #dependencies
  -- if there is a performance problem.
  CREATE TABLE #nodes
  (
    nodeid            NVARCHAR(255),
    generation        INT,
    cid               ClassID,
    id                ObjID,
    label             NVARCHAR(255),
    name              ObjName,
    alertstateid      INT,
    resid             NVARCHAR(8),
    hasownershipnote  TINYINT,
    priorityid        INT,
    numchildren       INT,
    phy_cid           ClassID
  )
  INSERT #nodes SELECT node, gen, src_cid, src_id, label, '', 1, '', 0, 0, 0, phy_cid FROM #dependencies d, obj_class c WHERE c.cid = d.phy_cid

  -- Update the name and resid columns by cursoring over the distinct set of classes represented
  -- in #dependencies

  DECLARE dc CURSOR FOR SELECT DISTINCT d.cid, c.cname, pc.cid, pc.cname FROM #nodes d, obj_class c, obj_class pc WHERE c.cid = d.cid AND pc.cid = phy_cid
  OPEN dc
  DECLARE @cname NVARCHAR(24), @phy_cid ClassID, @phy_cname NVARCHAR(24)
  FETCH NEXT FROM dc INTO @cid, @cname, @phy_cid, @phy_cname
  WHILE @@fetch_status <> -1
  BEGIN

    EXEC("
      UPDATE n SET name = v.name, alertstateid=_AlertStateID, priorityid=_PriorityID, hasownershipnote=_HasOwnershipNote 
      FROM #nodes AS n, " + @cname + "_V v
      WHERE n.cid='" + @cid + "' AND v.id = n.id
    ")
    EXEC("
      UPDATE #nodes SET resid = _LargeIconName FROM " + @phy_cname + "_S WHERE phy_cid = '" + @phy_cid + "'
    ")
    FETCH NEXT FROM dc INTO @cid, @cname, @phy_cid, @phy_cname
  END
  DEALLOCATE dc

  --if delete flag is 1 
  IF @del_flag=1
   BEGIN
	DECLARE @del_count INT, @p_cid ClassID, @p_id ObjID, @c_cid ClassID, @c_id ObjID
	SELECT @del_count=0, @c_cid=@phy_obj_cid, @c_id=@phy_obj_id
	WHILE @del_count <>@ins_count
	BEGIN
	  SELECT @p_cid=src_cid, @p_id=src_id from containment where dst_cid=@c_cid and dst_id=@c_id
	  DELETE containment WHERE dst_cid=@c_cid and dst_id=@c_id and src_cid=@p_cid and src_id=@p_id
	  SELECT @c_cid=@p_cid, @c_id=@p_id
	  SELECT @del_count=@del_count+1
	END
   END
-- end of del_flag=1


  SELECT nodeid, generation, cid, id, label, name, alertstateid, resid, hasownershipnote, priorityid, numchildren, phy_cid, 'deleted' = 0 FROM #nodes ORDER BY nodeid
