-- $Header:   //ARCHIVE/PVCSROOT/Archive_NLS/db/sql/_GetManagedObjects.sqv   1.0   Aug 22 2001 11:14:48   Dai  $
--
-- _GetManagedObjects.sql
--
-- Module Description
--   Defines the _GetManagedObjects stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- _GetManagedObjects
PRINT 'Creating procedure _GetManagedObjects'
GO

IF EXISTS(SELECT * FROM sysobjects WHERE name = '_GetManagedObjects' AND type = 'P')
  DROP PROC _GetManagedObjects
GO
-- $Archive:   //ARCHIVE/PVCSROOT/Archive_NLS/db/sql/_GetManagedObjects.sqv  $
-- $Revision:   1.0  $
-- $Date:   Aug 22 2001 11:14:48  $
-- $Author:   Dai  $
CREATE PROCEDURE _GetManagedObjects
  @cid              ClassID,
  @id               ObjID,
  @LocationPattern  NVARCHAR(255) = NULL,
  @NamePattern      NVARCHAR(255) = NULL,
  @MinAlert         INT = 0,
  @MinPriority      INT = 0,
  @OpenNotes        INT = 2,
  @ObjectType       INT = 0
AS
  --08/07/2001, still keep the limitation, but the customer can configure this value

  -- search limitation
  DECLARE @DESCENDENTS_LIMIT INT
  SELECT @DESCENDENTS_LIMIT = CONVERT(INT, propertyvalue) 
  FROM SystemConfiguration
  WHERE groupname = '_GetManagedObjects' AND propertyname = 'DESCENDENTS_LIMIT'
  IF @DESCENDENTS_LIMIT IS NULL OR @DESCENDENTS_LIMIT < 0
    SET @DESCENDENTS_LIMIT = 10000

  -- display limitation
  DECLARE @RETURN_LIMIT INT
  SELECT @RETURN_LIMIT = CONVERT(INT, propertyvalue) 
  FROM SystemConfiguration
  WHERE groupname = '_GetManagedObjects' AND propertyname = 'RETURN_LIMIT'
  IF @RETURN_LIMIT IS NULL OR @RETURN_LIMIT < 0
    SET @RETURN_LIMIT = 2000

  IF @DESCENDENTS_LIMIT < @RETURN_LIMIT
    SET @DESCENDENTS_LIMIT = @RETURN_LIMIT

  SET ROWCOUNT @DESCENDENTS_LIMIT


  -- Get the list of managed objects under the calling object
  CREATE TABLE #descendent
  (
    cid   ClassID PRIMARY KEY (cid, id),
    id    ObjID,
    gen   INT,
  )

  DECLARE @where NVARCHAR(255)
  SET @where = 'IN (SELECT cid FROM isa_chain WHERE base_cid = ''MNO2'')'

  -- IF object type was specified, DELETE anything that doesn't match
  IF @ObjectType <> 0
  BEGIN
    SELECT @where = 'IN (''' + val_cid + ''')'
      FROM enum_value AS ev JOIN enumeration AS e ON ev.eid = e.eid
      WHERE e.name = 'ManagedObjectTypes' AND ev.value = @ObjectType
  END

  EXEC asisp_descendents @cid, @id, @where

  -- Populate TABLE with all managed objects under the calling object
  CREATE TABLE #obj
  (
    src_cid            ClassID NULL,
    src_id             ObjID NULL,
    cid                ClassID,
    id                 ObjID,
    Location           NVARCHAR(255) NULL,
    name               ObjName NULL,
    _AlertStateID      INT NULL,
    _PriorityID        INT NULL,
    _CurrentStateID    INT NULL,
    _DesiredStateID    INT NULL,
    _HasOwnershipNote  TINYINT NULL,
    Description        NVARCHAR(255) NULL
  )
  INSERT #obj
    SELECT c.src_cid, c.src_id, dst_cid, dst_id, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
    FROM #descendent AS d 
      JOIN link AS c ON c.dst_cid = d.cid AND c.dst_id = d.id AND c.deleted = 0
      JOIN link_type AS lt ON lt.link_type = c.link_type
    WHERE
      (lt.flag_containment=1 OR lt.flag_parent_child=1)

  -- Fill IN the attribute details FOR each object
  DECLARE @obj_cid ClassID, @cname NVARCHAR(24)
  DECLARE cc INSENSITIVE CURSOR FOR SELECT DISTINCT obj.cid, cname FROM #obj AS obj JOIN obj_class AS c ON c.cid = obj.cid
  OPEN cc
  FETCH NEXT FROM cc INTO @obj_cid, @cname
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    EXEC("
    UPDATE obj SET 
      name = o.name,
      Location = o.name,
      _AlertStateID = o._AlertStateID,
      _PriorityID = o._PriorityID,
      _CurrentStateID = o._CurrentStateID,
      _DesiredStateID = o._DesiredStateID,
      _HasOwnershipNote = o._HasOwnershipNote,
      Description = o._Desc
    FROM #obj AS obj, " + @cname + "_V AS o
    WHERE o.id = obj.id AND obj.cid = '" + @obj_cid + "'
    ")
    FETCH NEXT FROM cc INTO @obj_cid, @cname
  END
  DEALLOCATE cc

  -- Remove objects that do NOT match the filter criteria
  IF @NamePattern IS NOT NULL
    DELETE #obj WHERE name NOT LIKE @NamePattern
  IF @MinAlert <> 0
    DELETE #obj WHERE _AlertStateID < @MinAlert
  IF @MinPriority <> 0
    DELETE #obj WHERE _PriorityID > @MinPriority -- yes, priorities work IN reverse!
  IF @OpenNotes <> 2
    DELETE #obj WHERE _HasOwnershipNote <> @OpenNotes

  DECLARE @rc INT, @src_cid ClassID, @loop INT
  SELECT @rc = COUNT(*) FROM #obj
  SET @cname = ''
  SET @loop = 0
  WHILE @rc > 0 AND @loop < 10
  BEGIN
    DECLARE srcc INSENSITIVE CURSOR FOR SELECT DISTINCT src_cid, cname FROM #obj JOIN obj_class AS c ON c.cid = src_cid
    OPEN srcc
    FETCH NEXT FROM srcc INTO @src_cid, @cname
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      EXEC("
      UPDATE obj SET src_cid = c.src_cid, src_id = c.src_id, Location = parent.name + '/' + Location
      FROM #obj AS obj 
      JOIN " + @cname + "_C AS parent ON parent.id = obj.src_id
      LEFT OUTER JOIN link AS c ON c.dst_cid = obj.src_cid AND c.dst_id = obj.src_id
        JOIN link_type AS lt ON lt.link_type = c.link_type
   WHERE obj.src_cid = '" + @src_cid + "' AND obj.src_cid <> 'BUSC'
        AND (lt.flag_containment = 1 OR lt.flag_parent_child = 1)
      ")
      FETCH NEXT FROM srcc INTO @src_cid, @cname
    END
    DEALLOCATE srcc
    SELECT @rc = COUNT(*) FROM #obj WHERE src_cid <> 'ENT' AND src_cid IS NOT NULL
    SET @loop = @loop + 1
  END

  -- Filter out paths that don't match parameter, IF specified
  IF @LocationPattern IS NOT NULL
    DELETE #obj WHERE Location NOT LIKE @LocationPattern

  --08/07/2001, still keep the limitation here, but customer can configure this value

  SET ROWCOUNT @RETURN_LIMIT

  -- obtain the localized strings
  DECLARE @local_string NVARCHAR(255)
  CREATE TABLE #localized_strings
  (
    key_name         sysname,
    english_string   VARCHAR(128),
    local_string     NVARCHAR(255) NULL DEFAULT NULL
  )
  INSERT #localized_strings(key_name, english_string) SELECT '_GetManagedObjects', 'Location'
  INSERT #localized_strings(key_name, english_string) SELECT '_GetManagedObjects', 'Type'
  INSERT #localized_strings(key_name, english_string) SELECT '_GetManagedObjects', 'name'
  INSERT #localized_strings(key_name, english_string) SELECT '_GetManagedObjects', 'Description'
  EXEC asisp_getLocalizedString NULL, NULL, @local_string OUTPUT, '#localized_strings'

  -- store the localized strings in variables
  DECLARE @location_heading NVARCHAR(255), @type_heading NVARCHAR(255), @name_heading NVARCHAR(255), @description_heading NVARCHAR(255)
  SELECT @location_heading=local_string FROM #localized_strings WHERE key_name='_GetManagedObjects' AND english_string='Location'
  SELECT @type_heading=local_string FROM #localized_strings WHERE key_name='_GetManagedObjects' AND english_string='Type'
  SELECT @name_heading=local_string FROM #localized_strings WHERE key_name='_GetManagedObjects' AND english_string='name'
  SELECT @description_heading=local_string FROM #localized_strings WHERE key_name='_GetManagedObjects' AND english_string='Description'

  DROP TABLE #localized_strings

  -- RETURN results
  EXEC('
    SELECT
      obj._AlertStateID,
      obj._HasOwnershipNote,
      obj.Location AS N'''+@location_heading+''',
      c.label AS N'''+@type_heading+''',
      name AS N'''+@name_heading+''',
      obj._PriorityID,
      obj._CurrentStateID,
      obj._DesiredStateID,
      Description AS N'''+@description_heading+''',
      obj.cid,
      obj.id
    FROM
      #obj AS obj
        JOIN obj_class AS c ON c.cid = obj.cid
    ORDER BY
      obj.Location
  ')

