-- $Header:   //ARCHIVE/PVCSROOT/Archive_NLS/DB/sql/asisp_createMADRByCidId.sqv   1.1   Aug 22 2001 17:43:36   Forthman  $
-- asisp_createMADRByCidId
--
-- Module Description
--   stored procedure asisp_createMADRByCidId

Use Object
GO

-----------------------------------------------------------
PRINT 'create procedure asisp_createMADRByCidId'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'asisp_createMADRByCidId' AND type = 'P' )
  DROP PROCEDURE asisp_createMADRByCidId
GO

-- Goal: Create a MADR object if needed for a pre-existing cid, id 
-- Note this only works for the following Cids: D2SS, D2DS, D2LK, D2MS, D2PM, D2SP
CREATE PROCEDURE asisp_createMADRByCidId
  @cid ClassID,
  @id  ObjID
AS
  -- Normal transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_createMADRByCidId
  ELSE
    SAVE TRANSACTION asisp_createMADRByCidId

  SET NOCOUNT ON

  RAISERROR( 'asisp_createMADRByCidId @cid = ''%s'', @id = %d', 0, 1, @cid, @id)

  -- if MADR instance does not exist then create a MADR instance 
  IF NOT EXISTS ( SELECT * FROM MVSAddressSpace_V WHERE _InstanceCID = @cid AND _InstanceID = @id AND deleted = 0)
  BEGIN
    DECLARE @name                          ObjName
    DECLARE @parent_cid                    ClassID
    DECLARE @parent_id                     ObjID
    DECLARE @InstanceCID                   NVARCHAR(4)
    DECLARE @InstanceID                    INT 
    DECLARE @SMFID                         NVARCHAR(4) 

    -- @name
    DECLARE @tablename NVARCHAR(256)
    SELECT  @tablename = cname + '_V' FROM obj_class WHERE cid = @cid

    CREATE TABLE #name 
    (
      name ObjName
    )

    DECLARE @sql NVARCHAR(2000)
    SELECT  @sql = '
    INSERT INTO #name SELECT name FROM ' + @tablename + ' WHERE id = ' + CONVERT(NVARCHAR(10), @id)
    EXEC ( @sql )

    SELECT @name = name FROM #name
    DROP TABLE #name

    -- @parent_cid
    SET @parent_cid = 'OS'
    
    -- @parent_id
    -- NOTE: This is the section of the code only works for the following cids: D2SS, D2DS, D2LK, D2MS, D2PM, D2SP
    IF @cid = 'D2SS' 
    BEGIN
      -- D2SS
      SELECT @parent_id = src_id 
      FROM link
      WHERE 
            src_cid = @parent_cid
        AND dst_cid = @cid
        AND dst_id  = @id
        AND deleted = 0
    END
    ELSE 
    BEGIN
      -- D2DS, D2LK, D2MS, D2PM, D2SP
      SELECT @parent_id = l.src_id
      FROM link AS l
      INNER JOIN link AS d2ss ON d2ss.src_cid = l.dst_cid AND d2ss.src_id = l.dst_id
      WHERE 
            l.src_cid    = @parent_cid
        AND d2ss.dst_cid = @cid
        AND d2ss.dst_id  = @id
        AND l.deleted    = 0
        AND d2ss.deleted = 0
    END

    -- @InstanceCID 
    SET @InstanceCID = @cid

    -- @InstanceID 
    SET @InstanceID = @id
  
    -- @SMFID
    SELECT @SMFID = _SMFID FROM OS_V WHERE id = @parent_id

    -- IF all the parameters exist then either update or call asisp_createMADR
    IF    @name IS NOT NULL
      AND @parent_cid IS NOT NULL
      AND @parent_id IS NOT NULL
      AND @InstanceCID IS NOT NULL
      AND @InstanceID IS NOT NULL
      AND @SMFID IS NOT NULL
    BEGIN
      UPDATE MVSAddressSpace_C
      SET
        name         = @name,
        _ParentCID   = @parent_cid,
        _ParentID    = @parent_id,
        _InstanceCID = @InstanceCID,
        _InstanceID  = @InstanceID,
        _SMFID       = @SMFID,
        deleted      = 0
      WHERE
            name         = @name
        AND _ParentCID   = @parent_cid
        AND _ParentID    = @parent_id
      IF @@ROWCOUNT = 0 
      BEGIN 
        EXEC asisp_createMADR @name = @name, @parent_cid = @parent_cid, @parent_id = @parent_id, 
             @InstanceCID = @InstanceCID, @InstanceID = @InstanceID, @SMFID = @SMFID
      END
    END
  END

  -- Normal transaction logic
  IF @tcount = 0
    COMMIT TRANSACTION asisp_createMADRByCidId
GO