-- $Header:   //ARCHIVE/PVCSROOT/Archive_NLS/DB/sql/asisp_createMSGD.sqv   1.1   Sep 12 2001 14:27:16   Forthman  $
-- asisp_createMSGD.sql
-- 
-- Module Description
--   creates procedure asisp_createMSGD
--

Use Object
GO


------------------------------------------------------------
PRINT 'create procedure asisp_createMSGD'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'asisp_createMSGD' AND type = 'P' )
  DROP PROCEDURE asisp_createMSGD
GO

-- $Archive:   //ARCHIVE/PVCSROOT/Archive_NLS/DB/sql/asisp_createMSGD.sqv  $
-- $Revision:   1.1  $
-- $Date:   Sep 12 2001 14:27:16  $
-- $Author:   Forthman  $
CREATE PROCEDURE asisp_createMSGD
  @name                          ObjName,
  @parent_cid                    ClassID = '',
  @parent_id                     ObjID = 0,
  @AlertStateID                  INT = 1,
  @ChildLookupPattern            NVARCHAR(255) = "",
  @Desc                          NVARCHAR(255) = "",
  @Flags                         INT = 0,
  @MsgID                         NVARCHAR(255) = "",
  @PriorityID                    INT = 3,
  @ProcessingPattern             NVARCHAR(255) = "",
  @SearchExpression              NVARCHAR(255) = "",
  @SourceID                      NVARCHAR(8) = "User",
  @StateID                       INT = 0
AS
-- RAISERROR('asisp_createMSGD ''%s'', ''%s'', %d', 0, 1, @name, @parent_cid, @parent_id)
  -- Standard TRANSACTION logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN asisp_createMSGD
  ELSE
    SAVE TRAN asisp_createMSGD
  -- Allocate an id FOR the new object
  DECLARE @id ObjID
  EXEC @id = alloc_MSGD
  IF @id = 0
  BEGIN
    ROLLBACK TRAN asisp_createMSGD
    RETURN @id
  END
  DECLARE @r INT
  -- INSERT class record
  INSERT MessageDescription_C_BASE
  (
    id,
    name
  )
  SELECT
    @id,
    @name
  IF @@ROWCOUNT <> 1
  BEGIN
    ROLLBACK TRAN asisp_createMSGD
    RETURN -30000
  END
  INSERT MessageDescription_C_LOCAL
  (
    id,
    langid,
    description
  )
  SELECT
    @id,
    @@LANGID,
    NULL
  IF @@ROWCOUNT <> 1
  BEGIN
    ROLLBACK TRAN asisp_createMSGD
    RETURN -30000
  END
 
  -- INSERT attribute vector
  DECLARE @id_str NVARCHAR(9)
  SELECT @id_str = CONVERT(NVARCHAR(9), @id)
  INSERT MessageDescription_A_BASE
  (
    id,
    _AlertStateID,
    _Flags,
    _MsgID,
    _PriorityID,
    _SourceID,
    _StateID,
    _ParentCID,
    _ParentID
  )
  SELECT
    @id,
    @AlertStateID,
    @Flags,
    @MsgID,
    @PriorityID,
    @SourceID,
    @StateID,
    @parent_cid,
    @parent_id
  IF @@ROWCOUNT <> 1
  BEGIN
    ROLLBACK TRAN asisp_createMSGD
    RETURN -30000
  END
  INSERT MessageDescription_A_LOCAL
  (
    id,
    langid,
    _Desc,
    _ChildLookupPattern,
    _ProcessingPattern,
    _SearchExpression
  )
  SELECT
    @id,
    @@LANGID,
    @Desc,
    @ChildLookupPattern,
    @ProcessingPattern,
    @SearchExpression
  IF @@ROWCOUNT <> 1
  BEGIN
    ROLLBACK TRAN asisp_createMSGD
    RETURN -30000
  END

  -- Any automatically created children are created here
  DECLARE @child_id ObjID
  
  -- Notify dispatcher of new message description
  EXEC asisp_notifyMessageDescription 20 /* MessageDescriptionInserted */, @id

  -- Repopulate the MessageDescription_T TABLE FROM the MessageDescription VIEW
  DELETE MessageDescription_T_LOCAL
    WHERE 
          obj_cid = @parent_cid
      AND obj_id  = @parent_id
      AND MsgID   = @MsgID
  DELETE MessageDescription_T_BASE
    WHERE 
          obj_cid = @parent_cid
      AND obj_id  = @parent_id
      AND MsgID   = @MsgID
  INSERT MessageDescription_T_BASE(obj_cid, obj_id, msgd_id, MsgID, StateID, AlertStateID, PriorityID, Flags)
    SELECT obj_cid, obj_id, msgd_id, MsgID, StateID, AlertStateID, PriorityID, Flags
      FROM MessageDescription
        WHERE 
              obj_cid = @parent_cid
          AND obj_id  = @parent_id
          AND MsgID   = @MsgID
  INSERT MessageDescription_T_LOCAL(obj_cid, obj_id, MsgID, langid, State, AlertState, Priority, 
            SearchExpression, ChildLookupPattern, ProcessingPattern, Description)
    SELECT obj_cid, obj_id, MsgID, @@LANGID, State, AlertState, Priority, 
            SearchExpression, ChildLookupPattern, ProcessingPattern, Description
      FROM MessageDescription
        WHERE 
              obj_cid = @parent_cid
          AND obj_id  = @parent_id
          AND MsgID   = @MsgID

  -- Finish up TRANSACTION
  IF @tcount = 0
    COMMIT TRAN asisp_createMSGD
  RETURN @id
GO