-- $Header:   //ARCHIVE/PVCSROOT/Archive_NLS/db/sql/asisp_descendents.sqv   1.2   Aug 21 2001 15:35:42   Dai  $
--
-- asisp_descendents.sql
--
-- Module Description
--   Defines the asisp_descendents stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- asisp_descendents
PRINT 'Creating procedure asisp_descendents'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_descendents' AND type = 'P')
  DROP PROCEDURE asisp_descendents
GO

-- $Archive:   //ARCHIVE/PVCSROOT/Archive_NLS/db/sql/asisp_descendents.sqv  $
-- $Revision:   1.2  $
-- $Date:   Aug 21 2001 15:35:42  $
-- $Author:   Dai  $
/* 
  Returns all descendents of the calling object of the class OR classes specified by a lookup
  expression.
  Caller must CREATE a TABLE to hold the results, AS follows
  CREATE TABLE #descendent
  (
    cid   ClassID,
    id    ObjID,
    gen   INT
  )
 */
CREATE PROCEDURE asisp_descendents
  @cid            ClassID,
  @id             ObjID,
  @desc_cid_expr  NVARCHAR(1000)=NULL, -- Descendent class id expression. If specified, must be one of the following forms:
                                       --   'IN (<CID1>[,<CID2>...])' OR 'LIKE <wildcard>'
  @verbose        TINYINT=0  -- Specify @verbose=1 to display diagnostic messages during execution
AS
  SET NOCOUNT ON

  IF @verbose = 1
  BEGIN
    PRINT CONVERT(NVARCHAR(20), CURRENT_TIMESTAMP, 120) + ': asisp_descedents: begin'
    PRINT 'asisp_descendents @cid=''' + @cid + ''', @id=' + CAST(@id AS VARCHAR) 
      + ', @desc_cid_expr=' + CASE WHEN @desc_cid_expr IS NULL THEN 'NULL' ELSE '''' + REPLACE(@desc_cid_expr, '''', '''''')  + '''' END
  END

  DECLARE @link_chain_clause NVARCHAR(255)

  CREATE TABLE #desc
  (
   cid ClassID,
   id ObjID,
   gen INT,
   nonleaf TINYINT
  )

  IF @cid = 'LOB'
  BEGIN
    IF @desc_cid_expr IS NULL
      SET @link_chain_clause = ''
    ELSE
      SET @link_chain_clause = 'AND (phy_cid ' + @desc_cid_expr + ' OR phy_cid IN (SELECT src_cid FROM link_chain WHERE cid ' + @desc_cid_expr + ') )'

    -- Populate worktable with the descendent LOBs that are either linked directly to specified
    -- object classes, OR linked to physical objects that can contain objects of those types
    CREATE TABLE #lob
    (
      src_id  ObjID,
      dst_id  ObjID,
      gen     INT,
      phy_cid ClassID NULL,
      phy_id  ObjID,
      nonleaf INT
    )

    DECLARE @id_str NVARCHAR(9), @verbose_str NVARCHAR(10)
    SELECT @id_str = CONVERT(NVARCHAR(9), @id), @verbose_str = CAST(@verbose AS NVARCHAR(10))
    EXEC("
    DECLARE @cid ClassID, @id ObjID
    SELECT @cid = '" + @cid + "', @id = " + @id_str + "
    INSERT #lob
    SELECT l.src_id, l.dst_id, l.gen, l.phy_cid, l.phy_id, 0 AS nonleaf
    FROM lob_link l
    WHERE 
      src_id = @id
      " + @link_chain_clause + "
    INSERT #lob
    SELECT l.src_id, l.dst_id, l.gen, l.phy_cid, l.phy_id, 0 AS nonleaf
    FROM lob_link l
    WHERE 
      dst_id = @id
      " + @link_chain_clause + "
      AND NOT EXISTS (SELECT * FROM #lob AS prev WHERE prev.dst_id = l.dst_id)

    UPDATE parent SET nonleaf = 1
      FROM #lob AS parent 
        JOIN LOB_LOB_LOBC AS parent_child ON parent.dst_id = parent_child.src_id
        JOIN #lob AS child ON child.dst_id = parent_child.dst_id
    WHERE
      child.phy_cid " + @desc_cid_expr + "
    WHILE @@ROWCOUNT <> 0
    BEGIN
      IF " + @verbose_str + " = 1
        PRINT CONVERT(NVARCHAR(20), CURRENT_TIMESTAMP, 120) + ': asisp_descedents: trimming'
      UPDATE parent SET nonleaf = 1
        FROM #lob AS parent 
          JOIN LOB_LOB_LOBC AS parent_child ON parent.dst_id = parent_child.src_id
          JOIN #lob AS child ON child.dst_id = parent_child.dst_id
        WHERE
          parent.nonleaf = 0
          AND child.nonleaf = 1 -- deleted parent from previous generation
    END

    INSERT #desc SELECT DISTINCT phy_cid, phy_id, 0, nonleaf FROM #lob
    ")
  END
  ELSE
  BEGIN
    INSERT #desc SELECT @cid, @id, 0, 0
  END

  -- Now iterate through the #descendent table to build the list of all possible containers.
  DECLARE @descendent_delete_expr NVARCHAR(255), @diagnostic_expr NVARCHAR(255)
  IF @desc_cid_expr IS NULL
  BEGIN
    SET @link_chain_clause = ''
    SET @descendent_delete_expr = ''
  END
  ELSE
  BEGIN
    SET @link_chain_clause = 'AND l.cid ' + @desc_cid_expr
    SET @descendent_delete_expr = 'DELETE #desc WHERE cid NOT ' + @desc_cid_expr
  END
  IF @verbose = 1
    SET @diagnostic_expr = 'PRINT CONVERT(NVARCHAR(20), CURRENT_TIMESTAMP, 120) + '': asisp_descedents: gen='' + CAST(@gen AS NVARCHAR)'
  ELSE
    SET @diagnostic_expr = ''

  EXEC("
  DECLARE @gen INT
  SELECT @gen = 0
  WHILE @@ROWCOUNT > 0
  BEGIN
      SELECT @gen = @gen + 1
      " + @diagnostic_expr + "
      INSERT #desc 
        SELECT DISTINCT c.dst_cid, c.dst_id, @gen, 0
        FROM #desc AS d
          JOIN link AS c ON c.src_cid = d.cid AND c.src_id = d.id
          JOIN link_chain AS l ON l.dst_cid = c.dst_cid
        WHERE
          d.gen = @gen - 1
          AND c.deleted = 0
	  AND d.nonleaf = 0
          AND NOT EXISTS (SELECT * FROM #desc prev WHERE prev.cid = c.dst_cid AND prev.id = c.dst_id)
          " + @link_chain_clause + "
    END
    " + @descendent_delete_expr 
    )
  INSERT into #descendent SELECT cid, id, gen FROM #desc

GO
