-- $Header:   //ARCHIVE/Archive_NLS/DB/sql/asisp_discoverCPSMResource.sqv   1.2   Aug 23 2001 18:59:26   Xiaoyan  $
--
-- asisp_discoverCPSMResource.sql
--
-- Module Description
--   Defines the asisp_discoverCPSMResource stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- asisp_discoverCPSMResource
PRINT 'Creating procedure asisp_discoverCPSMResource'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_discoverCPSMResource' AND type = 'P')
  DROP PROCEDURE asisp_discoverCPSMResource
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/sql/asisp_discoverCPSMResource.sqv  $
-- $Revision:   1.2  $
-- $Date:   Aug 23 2001 18:59:26  $
-- $Author:   Xiaoyan  $
-- Creates and processes a discovery batch for CPSM using the specified parameters
CREATE PROCEDURE asisp_discoverCPSMResource
  @FormatType        SMALLINT,
  @ActionType        SMALLINT,
  @TimeStamp         NVARCHAR(40) = NULL,
  @SeqNo             NVARCHAR(16) = NULL,
  @cid               NVARCHAR(4)  = NULL,
  @SystemId          NVARCHAR(40) = NULL,
  @CICSPLEX          NVARCHAR(40) = NULL,
  @CMAS              NVARCHAR(40) = NULL,
  @CICSName          NVARCHAR(40) = NULL,
  @MASTYPE           NVARCHAR(40) = NULL,
  @JOBNAME           NVARCHAR(40) = NULL,
  @APPLID            NVARCHAR(40) = NULL,
  @MVSID             NVARCHAR(40) = NULL,
  @TranClass         NVARCHAR(40) = NULL,
  @TranId            NVARCHAR(40) = NULL,
  @FileName          NVARCHAR(40) = NULL,
  @DSName            NVARCHAR(40) = NULL,
  @REMOTENAME        NVARCHAR(40) = NULL,
  @REMOTESYSTEM      NVARCHAR(40) = NULL,
  @STATE1            NVARCHAR(40) = NULL,
  @STATE2            NVARCHAR(40) = NULL,
  @auto_create       TINYINT = 1,
  @obj_cid           ClassID OUTPUT,
  @obj_id            ObjID OUTPUT,
  @obj_name          ObjName OUTPUT
AS
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_discoverCPSMResource
  ELSE
    SAVE TRANSACTION asisp_discoverCPSMResource

  -- Create a discovery batch record
  DECLARE @now DATETIME
  SELECT @now = GETDATE()
  INSERT DiscoveryBatch(ctime, FormatType, ActionType, numrows) 
    SELECT @now, @FormatType, @ActionType, 1
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_discoverCPSMResource
    RETURN -30000
  END

  DECLARE @batchid INT
  SELECT @batchid = batchid FROM DiscoveryBatch WHERE ctime = @now
  IF @batchid IS NULL
  BEGIN
    ROLLBACK TRANSACTION asisp_discoverCPSMResource
    RETURN -30000
  END

  -- If this is a file or transaction then we need to insert a record for the containing region
  IF @cid IN ( 'CIFL', 'CIFR', 'CITL', 'CITR' )
    INSERT INTO CPSMDiscovery(batchid, FormatType, ActionType, TimeStamp, SeqNo, cid, SystemId, CICSPLEX, CMAS, CICSName, MASTYPE, JOBNAME, APPLID, MVSID, TranClass, TranId, FileName, REMOTENAME, REMOTESYSTEM, STATE1, STATE2)
      SELECT @batchid, @FormatType, @ActionType, @TimeStamp, @SeqNo, 'CIMA', @SystemId, @CICSPLEX, @CMAS, @CICSName, @MASTYPE, @JOBNAME, @APPLID, @MVSID, NULL, NULL, NULL, NULL, NULL, NULL, NULL

  INSERT INTO CPSMDiscovery(batchid, FormatType, ActionType, TimeStamp, SeqNo, cid, SystemId, CICSPLEX, CMAS, CICSName, MASTYPE, JOBNAME, APPLID, MVSID, TranClass, TranId, FileName, REMOTENAME, REMOTESYSTEM, STATE1, STATE2)
    SELECT @batchid, @FormatType, @ActionType, @TimeStamp, @SeqNo, @cid, @SystemId, @CICSPLEX, @CMAS, @CICSName, @MASTYPE, @JOBNAME, @APPLID, @MVSID, @TranClass, @TranId, @FileName, @REMOTENAME, @REMOTESYSTEM, @STATE1, @STATE2
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_discoverCPSMResource
    RETURN -30000
  END

  DECLARE @r INT
  EXEC @r = _DiscoverCPSMResources ROOT, 1, @batchid, @incremental=1

  SELECT @obj_cid = obj_cid, @obj_id = obj_id, @obj_name = obj_name
  FROM ProcessedCPSMDiscovery WHERE batchid = @batchid AND cid = @cid 

  IF @tcount = 0
    COMMIT TRANSACTION asisp_discoverCPSMResource

  RETURN @r
GO
