-- $Header:   //ARCHIVE/Archive_NLS/DB/SQL/asisp_processStagedSJMs.sqv   1.4   Aug 10 2001 09:39:20   BRENT  $
--
-- asisp_processStagedSJMs.sql
--
-- Module Description
--   Defines the asisp_processStagedSJMs stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- asisp_processStagedSJMs
PRINT 'Creating procedure asisp_processStagedSJMs'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_processStagedSJMs' AND type = 'P')
  DROP PROCEDURE asisp_processStagedSJMs
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/SQL/asisp_processStagedSJMs.sqv  $
-- $Revision:   1.4  $
-- $Date:   Aug 10 2001 09:39:20  $
-- $Author:   BRENT  $
CREATE PROCEDURE asisp_processStagedSJMs
  @rowlimit INT = 1000,
  @debug    TINYINT = 0
AS
  -- exit quickly if there is no data to process
  IF NOT EXISTS (SELECT * FROM StagedSJM)
  BEGIN
    RAISERROR('asisp_processStagedSJMs: no staged events to process', 0, 1)
    RETURN 0
  END

  RAISERROR('asisp_processStagedSJMs: rowcount=%d', 0, 1, @rowlimit)
  SET ROWCOUNT @rowlimit

  DECLARE @rc INT, @err INT, @msg NVARCHAR(255), @chkpt INT
  SELECT @chkpt = 1

  -- Create temp table with ids of all SJM events that are available for processing, 'available'
  -- being defined as a SJM whose parent object does NOT have any earlier MESG events that have
  -- not been processed. Note that not all fields from StagedSJM are included since they are not
  -- currently in use. These are indicated by the comments in the table definition.
  CREATE TABLE #available_sjm 
  (
    timestamp         BINARY(8) PRIMARY KEY,
    ctime             DATETIME,
    parent_cid        ClassID,
    parent_id         ObjID,
    DATE_TIME         DATETIME,
    SEQ_NO            NVARCHAR(80) NULL,
--    OBJECT_NAME       ObjName NULL,
    CHILD_OBJ_TYPE    ClassID NULL,
    CHILD_OBJ_NAME    ObjName NULL,
    MESSAGE           NVARCHAR(16) NULL,
    STATE             NVARCHAR(40) NULL,
    REASON_CD         NVARCHAR(255) NULL,
    DISCOVERY_IND     NVARCHAR(4) NULL, 
    PARM_1            NVARCHAR(128) NULL,
    PARM_2            NVARCHAR(128) NULL,
    PARM_3            NVARCHAR(128) NULL,
    PARM_4            NVARCHAR(128) NULL,
--    PARM_5            NVARCHAR(128) NULL,
--    PARM_6            NVARCHAR(128) NULL,
--    PARM_7            NVARCHAR(128) NULL,

    -- These are fields for holding the values extracted from the REASON_CD field
    field1            NVARCHAR(100) NULL,
    field2            NVARCHAR(100) NULL,
    field3            NVARCHAR(100) NULL,
    field4            NVARCHAR(100) NULL
  )
  INSERT #available_sjm
    SELECT
      timestamp,
      ctime,
      parent_cid,
      parent_id,
      DATE_TIME,
      SEQ_NO,
--      OBJECT_NAME,
      CHILD_OBJ_TYPE,
      CHILD_OBJ_NAME,
      MESSAGE,
      STATE,
      REASON_CD,
      DISCOVERY_IND,
      PARM_1,
      PARM_2,
      PARM_3,
      PARM_4,
--      PARM_5,
--      PARM_6,
--      PARM_7,
      NULL,
      NULL,
      NULL,
      NULL
    FROM StagedSJM f
    WHERE
      f.timestamp NOT IN (SELECT evt_timestamp FROM ProcessedStagedEvent)
      AND NOT EXISTS
      (
        SELECT * FROM StagedMESG m
        WHERE 
          m.parent_cid = f.parent_cid AND m.parent_id = f.parent_id
          AND m.timestamp < f.timestamp
          AND NOT EXISTS (SELECT * FROM ProcessedStagedEvent WHERE evt_timestamp = m.timestamp)
      )
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  EXEC master..xp_asisprintf @msg output, 'asisp_processStagedSJMs: @@ERROR=%d, @@ROWCOUNT=%d', @err, @rc
  EXEC asisp_checkpointmessage @msg, 1, @chkpt
  SELECT @chkpt = @chkpt + 1
  IF @err <> 0
  BEGIN
    RETURN -30000
  END
  IF @rc = 0
  BEGIN
    RETURN 0
  END

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_processStagedSJMs
  ELSE
    SAVE TRANSACTION asisp_processStagedSJMs

  EXEC asisp_checkpointmessage 'asisp_processStagedSJMs: TRAN BEGIN', 0, @chkpt
  IF @debug = 1
    SELECT @chkpt AS chkpt, * FROM #available_sjm
  SELECT @chkpt = @chkpt + 1

  -- Also put event timestamp records into a table we will use to insert ProcessStagedEvent
  -- when successfully finished
  SELECT timestamp INTO #processed_sjm FROM #available_sjm
  DECLARE @num_events_processed INT
  SELECT @num_events_processed = COUNT(*) FROM #processed_sjm
  EXEC asisp_checkpointmessage 'asisp_processStagedSJMs: inserting INTO StagedFMSG', 0, @chkpt
  SELECT @chkpt = @chkpt + 1

  -- There are two general categories of subjob messages:
  -- 1. Messages with the child object identified explicitly (usually in the CHILD_OBJ_NAME field)
  -- 2. Messages where the child object must be identified by pattern matching on the message text.
  --
  -- Messages of the first category tend to be (re)discovery messages and come in large sets.
  -- These we handle explicitly.

  -- Handle FILE FCT messages  
  INSERT StagedFMSG(timestamp, ctime, parent_cid, parent_id, DATE_TIME, SEQ_NO, MESSAGE, REASON_CD, FileName, BatchNo, FileEnabled, FileOpen, Dataset)
    SELECT
      timestamp,
      ctime,
      parent_cid,
      parent_id,
      DATE_TIME,
      SEQ_NO,
      MESSAGE,
      REASON_CD,
      CONVERT(NVARCHAR(8),CHILD_OBJ_NAME) AS FileName,
      CASE WHEN PARM_4 LIKE 'BATCH%' THEN CONVERT(INT, SUBSTRING(PARM_4, 6, LEN(PARM_4)-5)) ELSE 0 END AS BatchNo,
      CONVERT(NVARCHAR(3), PARM_1) AS FileEnabled,
      CONVERT(NVARCHAR(3), PARM_2) AS FileOpen,
      PARM_3 AS Dataset
    FROM
      #available_sjm
    WHERE
      CHILD_OBJ_TYPE = 'FILE'
      AND CHILD_OBJ_NAME IS NOT NULL
  IF @@ROWCOUNT > 0
    DELETE #available_sjm
    WHERE
      CHILD_OBJ_TYPE = 'FILE'
      AND CHILD_OBJ_NAME IS NOT NULL

  -- Handle TRAN discovery messages
  INSERT StagedTRANDISC(timestamp, ctime, parent_cid, parent_id, DATE_TIME, SEQ_NO, MESSAGE, REASON_CD, TranName, TranStatus, TranPriority, TranClass, ProgramID, TWASize)
    SELECT
      timestamp,
      ctime,
      parent_cid,
      parent_id,
      DATE_TIME,
      SEQ_NO,
      CONVERT(NVARCHAR(16),MESSAGE),
      REASON_CD,
      CONVERT(NVARCHAR(8),CHILD_OBJ_NAME) AS TranName,
      CONVERT(NVARCHAR(16),STATE)  AS TranStatus,
      CONVERT(INT, PARM_1) AS TranPriority,
      CONVERT(NVARCHAR(16),PARM_2) AS TranClass,
      CONVERT(NVARCHAR(8),PARM_3) AS ProgramID,
      CONVERT(INT, PARM_4) AS TWASize
    FROM
      #available_sjm
    WHERE
      CHILD_OBJ_TYPE = 'TRAN'
      AND CHILD_OBJ_NAME IS NOT NULL
  IF @@ROWCOUNT > 0
    DELETE #available_sjm
    WHERE
      CHILD_OBJ_TYPE = 'TRAN'
      AND CHILD_OBJ_NAME IS NOT NULL

  -- The remaining messages in #available_sjm now have to be pattern-matched in order to identify
  -- the child objects.
  DECLARE @n INT
  SELECT @n = COUNT(*) FROM #available_sjm
  SET @msg = 'asisp_processStagedSJMs: Performing pattern-matching ON ' + CAST(@n AS NVARCHAR(9)) + ' message(s)'
  EXEC asisp_checkpointmessage @msg, 0, @chkpt
  IF @debug = 1
    SELECT @chkpt AS chkpt, * FROM #available_sjm
  SELECT @chkpt = @chkpt + 1

  -- **** WARNING ****
  -- Output Parameters from OLE calls MUST be VARCHAR they will return NULL if they are NVARCHAR
  -- **** WARNING ****
  DECLARE @update_sql VARCHAR(255), @errtext VARCHAR(255)

  -- Create a regular expression context for handling the extraction of data from the message text
  DECLARE @regex INT, @hr INT
  EXEC @hr = sp_OACreate 'TMRegExp.RegExContext', @regex OUT
  IF @hr <> 0
  BEGIN
    EXEC sp_OAGetErrorInfo NULL, NULL, @errtext OUT
    RAISERROR('%s', 3, 1, @errtext)
    ROLLBACK TRANSACTION asisp_processStagedSJMs
    RETURN -30000
  END

  -- For each distinct message, we look up its search expression and processing pattern from SJMessageMap
  DECLARE sjmmc INSENSITIVE CURSOR FOR
    SELECT DISTINCT mm.MsgID, mm.State, mm.SearchExpression, mm.ReplaceExpression
    FROM SJMessageMap mm JOIN #available_sjm AS a ON a.MESSAGE = mm.MsgID
  OPEN sjmmc
  DECLARE @MsgID NVARCHAR(16), @State NVARCHAR(40), @SearchExpression NVARCHAR(255), @ReplaceExpression NVARCHAR(255)
  FETCH NEXT FROM sjmmc INTO @MsgID, @State, @SearchExpression, @ReplaceExpression
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    -- Compile the regular expression into the processing context

    EXEC @hr = sp_OAMethod @regex, 'Compile', NULL, @SearchExpression, 0
    IF @hr <> 0
    BEGIN
      EXEC sp_OAGetErrorInfo @regex, NULL, @errtext OUT
      RAISERROR('%s', 3, 1, @errtext)
      INSERT INTO UnhandledStagedSJM SELECT
        CURRENT_TIMESTAMP,
        'Error compiling SearchExpression for MsgID [' + @MsgID + ']. Error: ' + @errtext,
        a.timestamp, a.ctime, a.parent_cid, a.parent_id, a.DATE_TIME, a.SEQ_NO, NULL, a.CHILD_OBJ_TYPE, a.CHILD_OBJ_NAME, a.MESSAGE, a.STATE, a.REASON_CD, a.DISCOVERY_IND, a.PARM_1, a.PARM_2, a.PARM_3, a.PARM_4, NULL, NULL, NULL
      FROM #available_sjm AS a
      WHERE MESSAGE = @MsgID
      DELETE #available_sjm WHERE MESSAGE = @MsgID
      GOTO NextEventType
    END

    -- Now cursor through each message of this type and perform the processing
    DECLARE sjmc CURSOR FOR SELECT timestamp, SEQ_NO, REASON_CD FROM #available_sjm WHERE MESSAGE = @MsgID
    OPEN sjmc
    DECLARE @timestamp BINARY(8), @SEQ_NO NVARCHAR(50), @REASON_CD NVARCHAR(255)
    FETCH NEXT FROM sjmc INTO @timestamp, @SEQ_NO, @REASON_CD
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      IF @debug = 1
      BEGIN
        PRINT 'Searching for pattern in text'
        PRINT '  PATTERN: ' + @SearchExpression
        PRINT '  NTEXT:    ' + @REASON_CD
      END
      -- Search the text
      EXEC @hr = sp_OAMethod @regex, 'Exec', NULL, @REASON_CD, 0
      IF @hr <> 0
      BEGIN
        EXEC sp_OAGetErrorInfo @regex, NULL, @errtext OUT
        RAISERROR('%s', 3, 1, @errtext)
        INSERT INTO UnhandledStagedSJM SELECT
          CURRENT_TIMESTAMP,
          'Error executing regular expression SEARCH. Error:' + @errtext,
          a.timestamp, a.ctime, a.parent_cid, a.parent_id, a.DATE_TIME, a.SEQ_NO, NULL, a.CHILD_OBJ_TYPE, a.CHILD_OBJ_NAME, a.MESSAGE, a.STATE, a.REASON_CD, a.DISCOVERY_IND, a.PARM_1, a.PARM_2, a.PARM_3, a.PARM_4, NULL, NULL, NULL
        FROM #available_sjm AS a
        WHERE a.timestamp = @timestamp
        DELETE #available_sjm WHERE timestamp = @timestamp
        GOTO NextEvent
      END

      -- Replace the matched values in the processing pattern
      DECLARE @processing_sql NVARCHAR(255)
      EXEC @hr = sp_OAMethod @regex, 'Replace', @update_sql OUT, @ReplaceExpression
      IF @hr <> 0
      BEGIN
        EXEC sp_OAGetErrorInfo @regex, NULL, @errtext OUT
        RAISERROR('%s', 3, 1, @errtext)
        INSERT INTO UnhandledStagedSJM SELECT
          CURRENT_TIMESTAMP,
          'Error executing regular expression REPLACE. Error:' + @errtext,
          a.timestamp, a.ctime, a.parent_cid, a.parent_id, a.DATE_TIME, a.SEQ_NO, NULL, a.CHILD_OBJ_TYPE, a.CHILD_OBJ_NAME, a.MESSAGE, a.STATE, a.REASON_CD, a.DISCOVERY_IND, a.PARM_1, a.PARM_2, a.PARM_3, a.PARM_4, NULL, NULL, NULL
        FROM #available_sjm AS a
        WHERE a.timestamp = @timestamp
        DELETE #available_sjm WHERE timestamp = @timestamp
        GOTO NextEvent
      END
      IF @debug = 1
      BEGIN
        PRINT '  STATE: ' + @State
        PRINT '  UPDATE_SQL: ' + @update_sql
      END
      -- Exec the processed SQL expression
      DECLARE @sql NVARCHAR(1000)
      SET @sql = 'UPDATE #available_sjm SET STATE=''' + @State + ''',' + @update_sql + ' WHERE CURRENT OF sjmc'
      EXEC(@sql)
      IF @@ERROR <> 0
      BEGIN
        INSERT INTO UnhandledStagedSJM SELECT
          CURRENT_TIMESTAMP,
          'Error executing SQL [' + @sql + ']. Error:' + @errtext,
          a.timestamp, a.ctime, a.parent_cid, a.parent_id, a.DATE_TIME, a.SEQ_NO, NULL, a.CHILD_OBJ_TYPE, a.CHILD_OBJ_NAME, a.MESSAGE, a.STATE, a.REASON_CD, a.DISCOVERY_IND, a.PARM_1, a.PARM_2, a.PARM_3, a.PARM_4, NULL, NULL, NULL
        FROM #available_sjm AS a
        WHERE a.timestamp = @timestamp
        DELETE #available_sjm WHERE timestamp = @timestamp
        GOTO NextEvent
      END
NextEvent:
      FETCH NEXT FROM sjmc INTO @timestamp, @SEQ_NO, @REASON_CD
    END
    DEALLOCATE sjmc
NextEventType:
    FETCH NEXT FROM sjmmc INTO @MsgID, @State, @SearchExpression, @ReplaceExpression
  END
  DEALLOCATE sjmmc

  SET @msg = 'asisp_processStagedSJMs: finished pattern-matching, inserting into StagedFILEMESG'
  EXEC asisp_checkpointmessage @msg, 0, @chkpt
  IF @debug = 1
    SELECT @chkpt AS chkpt, * FROM #available_sjm
  SELECT @chkpt = @chkpt + 1

  -- Insert into FILE AND TRAN specific staging tables for further processing
  INSERT StagedFILEMESG 
  (
    timestamp,
    ctime,
    parent_cid,
    parent_id,
    DATE_TIME,
    SEQ_NO,
    MESSAGE,
    STATE,
    REASON_CD,
    DISCOVERY_IND,
    FileName,
    Dataset
  )
  SELECT
    timestamp,
    ctime,
    parent_cid,
    parent_id,
    DATE_TIME,
    SEQ_NO,
    CONVERT(NVARCHAR(16),MESSAGE),
    CONVERT(NVARCHAR(40),STATE),
    REASON_CD,
    CONVERT(NVARCHAR(4),DISCOVERY_IND),
    CONVERT(NVARCHAR(8),field1),
    field2
  FROM #available_sjm
  WHERE CHILD_OBJ_TYPE = 'FILE'

  SET @msg = 'asisp_processStagedSJMs: INSERT INTO StagedTRANMESG'
  EXEC asisp_checkpointmessage @msg, 0, @chkpt
  IF @debug = 1
    SELECT @chkpt AS chkpt, * FROM #available_sjm
  SELECT @chkpt = @chkpt + 1

  INSERT StagedTRANMESG 
  (
    timestamp,
    ctime,
    parent_cid,
    parent_id,
    DATE_TIME,
    SEQ_NO,
    MESSAGE,
    STATE,
    REASON_CD,
    DISCOVERY_IND,
    TRANNAME,
    ABENDCODE,
    PROGRAMNAME,
    TERMCODE
  )
  SELECT
    timestamp,
    ctime,
    parent_cid,
    parent_id,
    DATE_TIME,
    SEQ_NO,
    CONVERT(NVARCHAR(16),MESSAGE),
    CONVERT(NVARCHAR(40),STATE),
    REASON_CD,
    CONVERT(NVARCHAR(4),DISCOVERY_IND),
    CONVERT(NVARCHAR(8),field1),
    CONVERT(NVARCHAR(16),field2),
    CONVERT(NVARCHAR(8),field3),
    CONVERT(NVARCHAR(4),field4)
   FROM #available_sjm 
  WHERE CHILD_OBJ_TYPE = 'TRAN'

  SET @msg = 'asisp_processStagedSJMs: calling asisp_processStagedFILEMESGs'
  EXEC asisp_checkpointmessage @msg, 0, @chkpt
  IF @debug = 1
    SELECT @chkpt AS chkpt, * FROM #available_sjm
  SELECT @chkpt = @chkpt + 1

  -- Call all of the detailed subjob message processors
  DECLARE @r INT

  EXEC @r = asisp_processStagedFILEMESGs
  IF @@ERROR <> 0 OR @r < 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedSJMs
    RETURN @r
  END
  ELSE
    DELETE StagedFILEMESG

  EXEC @r = asisp_processStagedTRANMESGs
  IF @@ERROR <> 0 OR @r < 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedSJMs
    RETURN @r
  END
  ELSE
    DELETE StagedTRANMESG

  EXEC @r = asisp_processStagedFMSGs @debug=@debug
  IF @@ERROR <> 0 OR @r < 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedSJMs
    RETURN @r
  END
  ELSE
    DELETE StagedFMSG

  SET @msg = 'asisp_processStagedSJMs: Calling asisp_processStagedTRANDISCs'
  EXEC asisp_checkpointmessage @msg, 0, @chkpt
  IF @debug = 1
    SELECT @chkpt AS chkpt, * FROM StagedTRANDISC
  SELECT @chkpt = @chkpt + 1
    
  EXEC @r = asisp_processStagedTRANDISCs @debug = @debug
  IF @@ERROR <> 0 OR @r < 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedSJMs
    RETURN @r
  END
  ELSE
    DELETE StagedTRANDISC

  -- If there are any IMIR events, process these
  IF EXISTS (SELECT * FROM #available_sjm WHERE CHILD_OBJ_TYPE = 'IMIR')
  BEGIN
    SET @msg = 'asisp_processStagedSJMs: Calling asisp_processStagedSJMIMIRs'
    EXEC asisp_checkpointmessage @msg, 0, @chkpt
    IF @debug = 1
      SELECT @chkpt AS chkpt, * FROM #available_sjm WHERE CHILD_OBJ_TYPE = 'IMIR'
    SELECT @chkpt = @chkpt + 1
    EXEC @r = asisp_processStagedSJMIMIRs
  END

  -- Mark events as processed
  INSERT ProcessedStagedEvent(evt_timestamp, ptime) 
    SELECT timestamp, GETDATE() FROM #processed_sjm
  EXEC asisp_checkpointmessage 'asisp_processStagedSJMs', 0, @chkpt
  SELECT @chkpt = @chkpt + 1

  -- Finish up transaction
  IF @tcount = 0
    COMMIT TRANSACTION asisp_processStagedSJMs

  RETURN @num_events_processed
GO
