-- $Header:   //ARCHIVE/Archive_NLS/DB/sql/asisp_processcontrolid.sqv   1.2   Aug 15 2001 17:44:34   Marc  $

USE Object
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_processcontrolid' AND type = 'P')
  DROP PROCEDURE asisp_processcontrolid
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/sql/asisp_processcontrolid.sqv  $
-- $Revision:   1.2  $
-- $Date:   Aug 15 2001 17:44:34  $
-- $Author:   Marc  $
CREATE PROCEDURE asisp_processcontrolid
  @cid ClassID,
  @id ObjID,
  @begintoken NVARCHAR(100) = '%this.',
  @endtoken NVARCHAR(100) = '%',
  @escape_char NVARCHAR(1) = '\',
  @control_id NVARCHAR(612) OUTPUT
AS

  -- standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN asisp_processcontrolid
  ELSE
    SAVE TRAN asisp_processcontrolid

  RAISERROR('asisp_processcontrolid: @cid=%s, @id=%d, @control_id=%s.', 0, 1, @cid, @id, @control_id)
  DECLARE @control_id_new NVARCHAR(612), @tokenlen INT, @endtokenlen INT, @escapelen INT, @sql NVARCHAR(2000), @escape_escape NVARCHAR(2), @escape NVARCHAR(100)
  SET @escape_escape = @escape_char + @escape_char
  SET @escape = @escape_char + @endtoken
  SET @control_id_new  = @control_id
  SET @tokenlen = LEN(@begintoken)
  SET @endtokenlen = LEN(@endtoken)
  SET @escapelen = LEN(@escape)

  -- temp table for results from dynamic SQL
  CREATE TABLE #results ( result NVARCHAR(612) )

  DECLARE @method MethodName, @method_string NVARCHAR(255), @param NVARCHAR(48), @param_type sysname, @param_len INT, @param_variable BIT, @param_string NVARCHAR(255)
  DECLARE @index INT, @index2 INT, @index3 INT, @index4 INT
  SET @index = CHARINDEX(@begintoken, @control_id)
  WHILE @index > 0
  BEGIN
    -- extract the current method name
    -- check for escape characters in the sequence
    SET @index2 = CHARINDEX(@endtoken, @control_id, @index+@tokenlen)
    SET @index4 = CHARINDEX(@escape, @control_id, @index+@tokenlen)
    WHILE (@index4 = @index2 OR (@index2-@index4 < @escapelen AND @index2-@index4 > 0)) AND @index4 > 0 AND @index2 > 0
    BEGIN
      SET @index3 = CHARINDEX(@escape, @control_id, @index2)
      SET @index2 = CHARINDEX(@endtoken, @control_id, @index2+@escapelen)
      IF @index3 > 0
        SET @index4 = @index3
    END
    -- check for spaces which indicate that the method has parameters
    SET @index3 = CHARINDEX(' ', @control_id, @index+@tokenlen)
    SET @index4 = @index2
    IF @index3 < @index2 AND @index3 > 0
      SET @index2 = @index3
    IF @index2 = 0
    BEGIN
      SET @index2 = @index+@tokenlen
      CONTINUE
    END
    SET @method = SUBSTRING(@control_id, @index + @tokenlen, @index2 - (@index + @tokenlen))
    SET @method_string = SUBSTRING(@control_id, @index + @tokenlen, @index4 - (@index + @tokenlen))
    -- replace the escape sequence with the end token sequence
    SET @index3 = CHARINDEX(@escape, @method_string)
    WHILE @index3 > 0
    BEGIN
        SET @method_string = SUBSTRING(@method_string, 1, @index3-1) + REPLACE(SUBSTRING(@method_string, @index3, @escapelen), @escape, @endtoken) + SUBSTRING(@method_string, @index3+@escapelen, LEN(@method_string)-(@index3+@escapelen)+1)
        IF @index3 + @escapelen - 1 <= LEN(@method)
          SET @method = SUBSTRING(@method, 1, @index3-1) + REPLACE(SUBSTRING(@method, @index3, @escapelen), @escape, @endtoken) + SUBSTRING(@method, @index3+@escapelen, LEN(@method)-(@index3+@escapelen)+1)
        SET @index3 = CHARINDEX(@escape, @method_string, @index3+@endtokenlen)
    END
    IF @method <> @method_string
      SET @method_string = @method_string + ','
    SET @method_string = '_' + @method_string

    -- make sure the method exists and can be executed on the object
    IF NOT EXISTS (SELECT * FROM method WHERE mname = @method) OR
      NOT EXISTS (SELECT * FROM obj_method om JOIN method m ON om.mid = m.mid JOIN isa_chain ic ON ic.base_cid = om.cid WHERE m.mname = @method AND ic.cid = @cid)
    BEGIN
      RAISERROR('asisp_processcontrolid: Error processing method %s for object %s, %d.', 11, 1, @method, @cid, @id)
      SET @index = CHARINDEX(@begintoken, @control_id, @index2+@endtokenlen)
      CONTINUE
    END

    -- verify the output params of the method
    IF (SELECT COUNT(*) FROM method_param mp JOIN method m ON m.mid = mp.mid WHERE m.mname = @method AND flag_output = 1) > 1 OR
      EXISTS (SELECT * FROM method_param mp JOIN method m ON m.mid = mp.mid WHERE m.mname = @method AND ((flag_collection = 1 AND flag_output = 1) OR (flag_output = 1 AND ptype <> 'ASIVARIANT')))
    BEGIN
      RAISERROR('asisp_processcontrolid: Parameters for method %s prohibit it from being called here.', 11, 1, @method, @cid, @id)
      SET @index = CHARINDEX(@begintoken, @control_id, @index2+@endtokenlen)
      CONTINUE
    END
    SELECT @param = pname FROM method m JOIN method_param mp ON mp.mid = m.mid WHERE m.mname = @method AND flag_output = 1

    -- find the native parameter type
    SELECT @param_type = st.name, @param_len = sc.prec, @param_variable = st.variable
    FROM method m JOIN method_param mp ON m.mid = mp.mid
      JOIN syscolumns sc ON OBJECT_ID('_' + m.mname) = sc.id AND sc.name = '@' + mp.pname
      JOIN systypes st ON sc.xtype = st.xtype
    WHERE m.mname = @method AND mp.pname = @param AND st.xusertype = st.xtype
    SET @param_string = 'DECLARE @result ' + @param_type
    IF @param_variable = 1
      SET @param_string = @param_string + '(' + CAST(@param_len AS NVARCHAR(10)) + ')'

    -- check for method overrides
    IF EXISTS (SELECT * FROM method m JOIN obj_method om ON m.mid = om.mid WHERE m.mname = @method AND om.cid = @cid AND om.flag_override = 1)
      SET @method_string = RTRIM(@cid) + @method_string

    -- execute the method
    RAISERROR('  executing method %s', 0, 1, @method)
    SET @sql = "
    SET NOCOUNT ON
    DELETE FROM #results
    " + @param_string + "
    EXEC " + @method_string + " @cid=" + @cid + ", @id=" + CAST(@id AS NVARCHAR(10)) + ", @" + @param + "=@result OUTPUT
    INSERT INTO #results SELECT CAST(@result AS NVARCHAR(255))
    "
    EXEC(@sql)
    IF @@ERROR <> 0
    BEGIN
      RAISERROR('asisp_processcontrolid: Error executing method %s.', 11, 1, @method)
      SET @index = CHARINDEX(@begintoken, @control_id, @index2+@endtokenlen)
      CONTINUE
    END

    -- replace the string with the result
    DECLARE @result NVARCHAR(255)
    SELECT @result = result FROM #results
    SET @control_id_new = REPLACE(@control_id_new, SUBSTRING(@control_id, @index, @index4 - @index + @endtokenlen), @result)

    SET @index = CHARINDEX(@begintoken, @control_id, @index2+@endtokenlen)
  END  
  RAISERROR('Old control_id: %s.', 0, 1, @control_id)
  SET @control_id = @control_id_new
  RAISERROR('New control_id: %s.', 0, 1, @control_id)

  -- finish transaction
  IF @tcount = 0
    COMMIT TRAN asisp_processcontrolid

GO