-- $Header:   //ARCHIVE/PVCSROOT/Archive_NLS/DB/sql/asisp_uploadGetOSJobs.sqv   1.1   Aug 24 2001 14:29:20   Forthman  $
-- asisp_uploadGetOSJobs.sql
--
-- Module Description
--   create procedure asisp_uploadGetOSJobs
--

Use Object
GO


-----------------------------------------------------------------------------
-- asisp_uploadGetOSJobs

PRINT 'create procedure asisp_uploadGetOSJobs'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'asisp_uploadGetOSJobs' AND type = 'P' )
  DROP PROCEDURE asisp_uploadGetOSJobs
GO

-- $Archive:   //ARCHIVE/PVCSROOT/Archive_NLS/DB/sql/asisp_uploadGetOSJobs.sqv  $
-- $Revision:   1.1  $
-- $Date:   Aug 24 2001 14:29:20  $
-- $Author:   Forthman  $
CREATE PROCEDURE asisp_uploadGetOSJobs
  @cid          ClassID,
  @id           ObjID,
  @target_os_id ObjID=NULL,   -- Useful when generating statements for only one OS
  @target_cid   ClassID=NULL, -- Useful when generating statements for all objects of a type
  @bGenerateMSGD TINYINT=1
as 
  RAISERROR('asisp_uploadGetOSJobs: %s, %d', 0, 1, @cid, @id)

  SET NOCOUNT ON

  DECLARE @sql NVARCHAR(255)

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@trancount
  IF @tcount = 0
    BEGIN TRANSACTION asisp_uploadGetOSJobs
  ELSE
    SAVE TRANSACTION asisp_uploadGetOSJobs

  -- TABLE of unique OS's that have jobs found
  -- by this query AND their associated plex IDs
  CREATE TABLE #os_list
  (
    os_id    ObjID,
    os_name  ObjName,
    plex_id  ObjID NULL DEFAULT NULL
  )
  -- TABLE of all OS's that are IN the same
  -- plexes as the OS's IN the set #os_list
  CREATE TABLE #os_plex_list
  (
    plex_id ObjID,
    os_id   ObjID,
    os_name ObjName
  )

  CREATE TABLE #uploadGetOSJobs
  (
    cid               ClassID,
    id                ObjID,
    name              ObjName      NULL DEFAULT NULL,
    _UploadUserID     NVARCHAR(16)  NULL DEFAULT NULL,
    _UploadPassword   NVARCHAR(16)  NULL DEFAULT NULL,
    svc_host          NVARCHAR(30)  NULL,
    svc_name          NVARCHAR(30)  NULL
  )

  IF @target_os_id is NOT null
    -- Useful when generating statements for only one OS
    INSERT #uploadGetOSJobs(cid, id, name, _UploadUserID, _UploadPassword, svc_host, svc_name) EXEC asisp_uploadGetOSInfo 'OS', @target_os_id
  ELSE
    INSERT #uploadGetOSJobs(cid, id, name, _UploadUserID, _UploadPassword, svc_host, svc_name) EXEC asisp_uploadGetOSInfo @cid, @id

  IF EXISTS (SELECT * FROM #uploadGetOSJobs)
  BEGIN
    RAISERROR('asisp_uploadGetOSJobs: all distinct OS''s found', 0, 1)
    SELECT DISTINCT cid, id, name, _UploadUserID, _UploadPassword, svc_host, svc_name
      FROM #uploadGetOSJobs
  END
  ELSE
  BEGIN
    RAISERROR('asisp_uploadGetOSJobs: WARNING - there are no MVS OS''s enabled for upload (_UploadEnabled). Halting processing.', 0, 1, @cid, @id)
    IF @tcount = 0
      COMMIT TRANSACTION asisp_uploadGetOSJobs
    RETURN 0
  END

  -- Special handling IF class derives FROM MREG (MVSRegisterable)
  IF exists (SELECT * FROM isa_chain WHERE cid=@cid AND base_cid='MREG' AND cid<>base_cid)
  BEGIN
    RAISERROR('asisp_uploadGetOSJobs: special handling as %s, %d is a MREG', 0, 1, @cid, @id)
    IF @bGenerateMSGD = 1
    BEGIN
      CREATE TABLE #upload_msgds
      (
        MsgID NVARCHAR(16)
      )

      INSERT #upload_msgds(MsgID)
        SELECT MsgID
        FROM MessageDescription_T
        WHERE obj_cid=@cid
          AND obj_id=0
          AND Flags&1=0  -- NOT MSGD_FLAG_NO_TRAP_REGISTER
      INSERT #upload_msgds(MsgID)
        SELECT MsgID
        FROM MessageDescription_T
        WHERE obj_cid=@cid
          AND obj_id=@id
          AND Flags&1=0  -- NOT MSGD_FLAG_NO_TRAP_REGISTER
          AND MsgID NOT IN (SELECT MsgID FROM #upload_msgds)

      SELECT 'MSGD_ROW|'+RTRIM(@cid)+'|'+CONVERT(NVARCHAR(255),@id)+'|'+MsgID FROM #upload_msgds

      DROP TABLE #upload_msgds
    END

    INSERT #os_list(os_id, os_name)
      SELECT DISTINCT id, name
        FROM #uploadGetOSJobs
    INSERT #os_plex_list exec asisp_getSysplexOSfromTemp

    -- output the data
    CREATE TABLE #jobinfo
    (
       cno    INT,
       cid    ClassID,
       id     ObjID,
       name   ObjName,
       _SysplexResource  TINYINT
    )
    INSERT #jobinfo(cno, cid, id, name, _SysplexResource)
      SELECT cno, j.cid, j.id, j.name, _SysplexResource
        FROM MVSJOB_C AS j
          JOIN obj_class AS oc ON j.cid=oc.cid
        WHERE 
          j.cid=@cid AND j.id=@id

    -- Create #outputDataRow in order to temporarily store the output to allow for manipulation of D2SS records
    CREATE TABLE #outputDataRow
    (
      os_id   ObjID,
      os_name ObjName,
      cno     INT,
      cid     ClassID,
      id      ObjID,
      name    ObjName
    )

    -- join to sysplex IF it IS a sysplex resource
    IF EXISTS (SELECT * FROM #jobinfo, #os_plex_list WHERE _SysplexResource=1)
    BEGIN
      RAISERROR('asisp_uploadGetOSJobs: MJOB is a SysplexResource.  Registering on all OS''s in the sysplex...', 0, 1)
      INSERT INTO #outputDataRow ( os_id, os_name, cno, cid, id, name )
      SELECT CONVERT(NVARCHAR(255),p.os_id), p.os_name, CONVERT(NVARCHAR(255),cno), RTRIM(@cid), CONVERT(NVARCHAR(255),@id), name
        FROM #jobinfo j, #os_plex_list p
    END
    ELSE
    BEGIN
      -- don't join to sysplex IF it IS NOT a sysplex resource
      RAISERROR('asisp_uploadGetOSJobs: MJOB is NOT a SysplexResource or os is not in a sysplex.', 0, 1)
      INSERT INTO #outputDataRow ( os_id, os_name, cno, cid, id, name )
      SELECT CONVERT(NVARCHAR(255),t.id), t.name, CONVERT(NVARCHAR(255),cno), RTRIM(@cid), CONVERT(NVARCHAR(255),@id), j.name
        FROM #uploadGetOSJobs AS t, MVSJOB_C AS j
          JOIN obj_class AS oc ON j.cid=oc.cid
        WHERE j.cid=@cid 
          AND j.id=@id
    END

    -- Fix: D2SS objects should use the name of their D2MS child because the D2MS (master) is considered the focus of a DB2 subsystem
    --      Sending the name of the child allows us to see the console traps for the D2MS yet allows a consistent GUI look and feel of 
    --      OS children having Source390 tabs in their property page. 
    UPDATE #outputDataRow 
    SET name = o.name
    FROM #outputDataRow AS j 
    INNER JOIN link AS l ON l.src_cid = j.cid AND l.src_id = j.id
    INNER JOIN DB2Master_V AS o ON o.id = l.dst_id
    WHERE 
          l.deleted = 0
      AND l.src_cid = 'D2SS'
      AND l.dst_cid = 'D2MS'


    -- output the results
    SELECT 'DATA_ROW|' + CONVERT(NVARCHAR(255),os_id) + '|' + os_name + '|' + CONVERT(NVARCHAR(255),cno) + '|' + RTRIM(cid) + '|' + CONVERT(NVARCHAR(255),id) + '|' + name
    FROM #outputDataRow

    RAISERROR('asisp_uploadGetOSJobs: total job registration count=%d', 0, 1, @@ROWCOUNT)

    DROP TABLE #outputDataRow
    DROP TABLE #jobinfo
    DROP TABLE #os_list
    DROP TABLE #os_plex_list

    IF @tcount = 0
      COMMIT TRANSACTION asisp_uploadGetOSJobs
    return 0
  END

  CREATE TABLE #uploadGetOSJobs2
  (
    os_id                ObjID,
    os_name              ObjName,
    cno                  INT,
    job_cid              ClassID,
    job_id               ObjID,
    job_name             ObjName,
    _SysplexResource     TINYINT
  )

  -- INSERT all of the jobs contained within the OS's found by asisp_uploadGetOSInfo
  INSERT #uploadGetOSJobs2(os_id, os_name, cno, job_cid, job_id, job_name, _SysplexResource)
    SELECT t.id, t.name, cno, j.cid, j.id, j.name, _SysplexResource
    FROM #uploadGetOSJobs t 
      JOIN link AS l ON src_cid=t.cid AND src_id=t.id
      JOIN MVSJOB_C AS j ON dst_cid=j.cid AND dst_id=j.id
      JOIN obj_class AS oc ON j.cid=oc.cid
    WHERE l.deleted=0
      AND _ConsoleTrapStatus=1

  -- cleanup TABLE as NOT used again
  DROP TABLE #uploadGetOSJobs

  -- Delete all rows FROM #uploadGetOSJobs2 that don't match the @target_cid.
  -- This is useful when generating statements for all objects of a type
  IF @target_cid is NOT null
    delete FROM #uploadGetOSJobs2 WHERE job_cid<>@target_cid

  INSERT #os_list(os_id, os_name)
    SELECT DISTINCT os_id, os_name
      FROM #uploadGetOSJobs2
  INSERT #os_plex_list exec asisp_getSysplexOSfromTemp

  RAISERROR('asisp_uploadGetOSJobs: all distinct OS''s found with plex_id', 0, 1)
  SELECT * FROM #os_plex_list

  CREATE TABLE #uploadGetOSJobs3
  (
    os_id                ObjID,
    os_name              ObjName,
    cno                  INT,
    job_cid              ClassID,
    job_id               ObjID,
    job_name             ObjName
  )

  -- INSERT all jobs for all OS's that are NOT participating IN a sysplex
  INSERT #uploadGetOSJobs3
    SELECT t.os_id, o.os_name, cno, job_cid, job_id, job_name
      FROM #uploadGetOSJobs2 AS t 
        JOIN #os_list AS o ON t.os_id=o.os_id
      WHERE plex_id IS NULL
  RAISERROR('asisp_uploadGetOSJobs: all jobs for all TARGET Non-Sysplex OS''s, @@ROWCOUNT=%d', 0, 1, @@ROWCOUNT)

  -- INSERT the jobs that DO NOT participate IN the sysplex 
  -- for TARGET OS's that DO participate IN sysplex
  INSERT #uploadGetOSJobs3
    SELECT o.os_id, o.os_name, cno, job_cid, job_id, job_name
      FROM #uploadGetOSJobs2 AS t 
        JOIN #os_list AS o ON t.os_id=o.os_id
      WHERE plex_id IS NOT NULL
        AND _SysplexResource=0
  RAISERROR('asisp_uploadGetOSJobs: non-sysplex jobs for all TARGET Sysplex OS''s, @@ROWCOUNT=%d', 0, 1, @@ROWCOUNT)

  -- Insert the jobs that DO participate IN the sysplex,
  -- for TARGET OS's that DO participate IN the sysplex
  -- Note: each job is sent to TARGET other all OTHER OS' IN the sysplex
  INSERT #uploadGetOSJobs3
    SELECT p.os_id, p.os_name, cno, job_cid, job_id, job_name
      FROM #os_plex_list p
      join #os_list o
        ON p.plex_id=o.plex_id
      join #uploadGetOSJobs2 t
        ON o.os_id=t.os_id
      WHERE _SysplexResource=1
  RAISERROR('asisp_uploadGetOSJobs: sysplex jobs for all TARGET Sysplex OS''s to be registered on TARGET and OTHER OS''s, @@ROWCOUNT=%d', 0, 1, @@ROWCOUNT)

  -- Insert the jobs that DO participate IN the sysplex,
  -- for all OTHER OS's that DO participate IN the sysplex
  -- except for the TARGET OS.
  -- (All sysplex jobs ON OTHER OS's IN the sysplex are
  -- sent to the TARGET OS)
  INSERT #uploadGetOSJobs3
    SELECT o.os_id, o.os_name, cno, j.cid, j.id, j.name
      FROM #os_plex_list AS p
       JOIN #os_list AS o ON p.plex_id=o.plex_id
       JOIN link AS l ON src_cid='OS' AND src_id=p.os_id
       JOIN MVSJOB_C AS j ON dst_cid=j.cid AND dst_id=j.id
       JOIN obj_class AS oc ON j.cid=oc.cid
      WHERE p.os_id<>o.os_id
        AND l.deleted=0
        AND _SysplexResource=1
  RAISERROR('asisp_uploadGetOSJobs: sysplex jobs for all OTHER Sysplex OS''s to be registered on all TARGET Sysplex OS''s, @@ROWCOUNT=%d', 0, 1, @@ROWCOUNT)

  IF @bGenerateMSGD = 1
  BEGIN
    CREATE TABLE #upload_msgds3
    (
      cid   ClassID,
      id    ObjID,
      MsgID NVARCHAR(255)
    )

    INSERT #upload_msgds3(cid, id, MsgID)
      SELECT DISTINCT job_cid, job_id, MsgID
        FROM MessageDescription_T t, #uploadGetOSJobs3 u
        WHERE obj_cid=job_cid
          AND obj_id=0
          AND Flags&1=0  -- NOT MSGD_FLAG_NO_TRAP_REGISTER
    INSERT #upload_msgds3(cid, id, MsgID)
      SELECT DISTINCT job_cid, job_id, MsgID
        FROM MessageDescription_T t, #uploadGetOSJobs3 u
        WHERE obj_cid=job_cid
          AND obj_id=job_id
          AND Flags&1=0  -- NOT MSGD_FLAG_NO_TRAP_REGISTER
          AND MsgID NOT IN (SELECT DISTINCT MsgID FROM #upload_msgds3 WHERE cid=job_cid AND id=job_id)

    SELECT 'MSGD_ROW|'+RTRIM(cid)+'|'+CONVERT(NVARCHAR(255),id)+'|'+MsgID+'|'
      FROM #upload_msgds3
      order by cid, id

    DROP TABLE #upload_msgds3
  END

  -- Create #outputDataRow2 in order to temporarily store the output to allow for manipulation of D2SS records
  CREATE TABLE #outputDataRow2
  (
    os_id   ObjID,
    os_name ObjName,
    cno     INT,
    cid     ClassID,
    id      ObjID,
    name    ObjName
  )

  -- Temporarily store the output data in #outputDataRow
  INSERT INTO #outputDataRow2 ( os_id, os_name, cno, cid, id, name )
  SELECT CONVERT(NVARCHAR(255),os_id), os_name, CONVERT(NVARCHAR(255),cno), RTRIM(job_cid), CONVERT(NVARCHAR(255),job_id), job_name
  FROM #uploadGetOSJobs3
  order by os_name, job_cid


  -- Fix: D2SS objects should use the name of their D2MS child because the D2MS (master) is considered the focus of a DB2 subsystem
  --      Sending the name of the child allows us to see the console traps for the D2MS yet allows a consistent GUI look and feel of 
  --      OS children having Source390 tabs in their property page. 
  UPDATE #outputDataRow2
  SET name = o.name
  FROM #outputDataRow2 AS j 
  INNER JOIN link AS l ON l.src_cid = j.cid AND l.src_id = j.id
  INNER JOIN DB2Master_V AS o ON o.id = l.dst_id
  WHERE 
        l.deleted = 0
    AND l.src_cid = 'D2SS'
    AND l.dst_cid = 'D2MS'

  -- output the results
  SELECT 'DATA_ROW|' + CONVERT(NVARCHAR(255),os_id) + '|' + os_name + '|' + CONVERT(NVARCHAR(255),cno) + '|' + RTRIM(cid) + '|' + CONVERT(NVARCHAR(255),id) + '|' + name
  FROM #outputDataRow2
  ORDER BY os_name, cid

  RAISERROR('asisp_uploadGetOSJobs: total job registration count=%d', 0, 1, @@ROWCOUNT)
  RAISERROR('asisp_uploadGetOSJobs: job registration counts by OS:', 0, 1)
  SELECT os_id, os_name, 'Count'=COUNT(*)
   FROM #uploadGetOSJobs3
   GROUP BY os_id, os_name

  DROP TABLE #outputDataRow2
  DROP TABLE #uploadGetOSJobs3
  DROP TABLE #os_list
  DROP TABLE #os_plex_list

  DROP TABLE #uploadGetOSJobs2

  -- Finish up transaction
  IF @tcount = 0
    COMMIT TRANSACTION asisp_uploadGetOSJobs
GO