PRINT 'Creating procedure cl_changeOnwership'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='cl_changeOnwership')
  DROP PROCEDURE cl_changeOnwership
GO

CREATE PROCEDURE cl_changeOnwership
@new_instrid NVARCHAR(255),
@old_instrid NVARCHAR(255) = '',
@where NVARCHAR(255) = ''
AS
  -- Changes the InstrumentationID of resources in CL_IDCache
  -- (which modifies the onwership of resources)
  -- Should specify @new_instrid and (@old_instrid or @where)
  RAISERROR('cl_changeOnwership: @new_instrid=%s, @old_instrid=%s, @where=%s', 0, 1, 
				 @new_instrid, @old_instrid, @where)
  SET NOCOUNT ON

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN cl_changeOnwership
  ELSE
    SAVE TRAN cl_changeOnwership

  IF @new_instrid IS NULL SET @new_instrid = ''
  IF @old_instrid IS NULL SET @old_instrid = ''
  IF @where IS NULL SET @where = ''


  IF @new_instrid = '' OR (@old_instrid = '' AND @where = '') BEGIN
    RAISERROR('Input should specify valid @new_instrid and (@old_instrid or @where)', 0, 1)
    ROLLBACK TRAN cl_changeOnwership
    RETURN -1
  END

  DECLARE @cnt INT, @sql NVARCHAR(255)
  IF @old_instrid <> '' BEGIN
    UPDATE CL_IDCache SET instrID=@new_instrid WHERE instrID=@old_instrid
    SET @cnt = @@ROWCOUNT
    RAISERROR('Updated CL_IDCache - [%s] took onwership of all resources onwed by [%s], [UpdateCount=%d]', 0, 1, @new_instrid, @old_instrid, @cnt)
  END
  ELSE BEGIN
    CREATE TABLE #rowcount ( cnt INT )
    SET @sql = 'UPDATE CL_IDCache SET instrID=''' + @new_instrid + ''' ' + @where + ' ' +
               'INSERT #rowcount SELECT @@ROWCOUNT'
    EXEC (@sql)
    IF @@ERROR = 0 BEGIN
      SELECT @cnt = cnt FROM #rowcount
      RAISERROR('Updated CL_IDCache - [%s] took onwership of all resources computed by where-clause [%s], [UpdateCount=%d]', 0, 1, @new_instrid, @where, @cnt)
    END
    ELSE BEGIN
      RAISERROR('Failed in dynamic sql call - [%s]', 0, 1, @sql)
      RAISERROR('Check your @where clause - WHERE should be inclusive in @where parameter', 0, 1, @sql)
      ROLLBACK TRAN cl_changeOnwership
      RETURN -1
    END
  END

  IF EXISTS (SELECT * FROM CL_Registration WHERE instrId=@new_instrid)
    RAISERROR('Located [%s] in CL_Registration, implies CL received registration message(s) from this InstrumentationID', 0, 1, @new_instrid)
  ELSE
    RAISERROR('Could not locate [%s] in CL_Registration, implies CL did not receive registration message(s) from this InstrumentationID yet!', 0, 1, @new_instrid)

  -- Finish up transaction
  IF @tcount = 0 
    COMMIT TRAN cl_changeOnwership

