PRINT 'Creating procedure cl_cleanStageTbls'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='cl_cleanStageTbls')
  DROP PROCEDURE cl_cleanStageTbls
GO

CREATE PROCEDURE cl_cleanStageTbls
AS
  -- Clean-up list:
  -- This proc cleans CL_StageBulkData, CL_StageDeltaData & CL_StageEvData

  RAISERROR('cl_cleanStageTbls: ', 0, 1)

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN cl_cleanStageTbls
  ELSE
    SAVE TRAN cl_cleanStageTbls

  -- stat is 1 if ERROR, else 0
  CREATE TABLE #transid (trid INT, stat INT)
  INSERT #transid SELECT DISTINCT trId, 0 FROM CL_Stage WHERE trState=5

  -- Get transactions that have one (or more) ERRORs
  UPDATE t SET t.stat=1 FROM #transid t JOIN CL_Status stat ON (t.trid=stat.trId)
    WHERE stat.status LIKE '%ERROR%'

  -- Store all data when transaction has ERRORs
  INSERT CL_Store SELECT s.trId, s.evDataId, s.action, s.root, s.keyword, s.value 
    FROM CL_StageBulkData s JOIN #transid t ON (t.trid=s.trId) WHERE t.stat=1

  INSERT CL_Store SELECT s.trId, s.evDataId, s.action, s.root, s.keyword, s.value 
    FROM CL_StageDeltaData s JOIN #transid t ON (t.trid=s.trId) WHERE t.stat=1

  INSERT CL_Store SELECT s.trId, s.evDataId, 0, s.root, s.keyword, s.value 
    FROM CL_StageEvData s JOIN #transid t ON (t.trid=s.trId) WHERE t.stat=1

  -- Clean stage tables
  DELETE a FROM CL_StageBulkData a, #transid b WHERE a.trId=b.trid
  DELETE a FROM CL_StageDeltaData a, #transid b WHERE a.trId=b.trid
  DELETE a FROM CL_StageEvData a, #transid b WHERE a.trId=b.trid

  IF @tcount = 0 
    COMMIT TRAN cl_cleanStageTbls
