PRINT 'Creating procedure cl_createResource'
GO

IF EXISTS(SELECT * FROM sysobjects WHERE type='P' AND name='cl_createResource')
  DROP PROCEDURE cl_createResource
GO

CREATE PROCEDURE cl_createResource
@cid ClassID,
@disp VARCHAR(8000) OUTPUT
AS  
  -- input in #attrs table
  RAISERROR('cl_createResource: @cid=%s, @disp OUT', 0, 1, @cid)

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN cl_createResource
  ELSE
    SAVE TRAN cl_createResource

  DECLARE @instid NVARCHAR(255), @oid ObjID
  SELECT @instid=value FROM #attrs WHERE root='inst' AND keyword='instid'

  -- cannot create resource if name and/or instid is invalid
  IF @instid IS NULL OR @instid = '' BEGIN
    SET @disp = @disp + ', InstanceID is not provided'
    ROLLBACK TRAN cl_createResource
    RETURN -1
  END
  SET @oid = 0

  -- Check if instid already exists
  SELECT @oid=oid FROM CL_IDCache 
    WHERE cid=@cid AND keyword='CL_InstID' AND value=@instid AND deleted=0
  IF @oid > 0 BEGIN
    ROLLBACK TRAN cl_createResource
    RETURN @oid
  END

  CREATE TABLE #inst_attrs (
    oid ObjID, 
    keyword nvarchar(64), 
    value nvarchar(255)
  )
  CREATE TABLE #bulkcreatetable (
    id ObjID DEFAULT 0,
    name NVARCHAR(255),
    work_int INT DEFAULT 0
  )

  DECLARE @createproc NVARCHAR(16), @instrid NVARCHAR(255), @createtime DATETIME
  INSERT #bulkcreatetable(name) SELECT @instid
  SET @createtime = GETDATE()
  SET @instrid = ''        

  -- Create resource
  SET @createproc = 'cl_process' + @cid
  EXEC @createproc @action='BULKCREATE'
  SELECT @oid=id FROM #bulkcreatetable WHERE name=@instid AND id>0
  IF @oid > 0 BEGIN	    
    -- get InstrumentationID
    SELECT @instrid=value FROM #attrs 
      WHERE root='CommonListener' AND keyword='instrId'

    -- Update Cache
    INSERT CL_IDCache (cid, oid, keyword, value, instrID)
      SELECT @cid, @oid, 'CL_InstID', @instid, @instrid 

    -- Update Attributes
    INSERT #inst_attrs SELECT @oid, keyword, value FROM #attrs 
      WHERE root='inst' OR root='inst:attr1'
    SET @createproc = 'cl_process' + @cid 
    EXEC @createproc @action='ADD_UPDATE'

    -- Notify that this class had a new resource created,
    -- so as to proces Link rules in CL_LinkRules
    IF NOT EXISTS (SELECT * FROM CL_ModifiedClasses WHERE cid=@cid)
      INSERT CL_ModifiedClasses SELECT @cid, @createtime

    -- LOB processing
    IF EXISTS (SELECT * FROM GEM_InstFiltering WHERE cid=@cid)
      INSERT GEM_UpdateLOBView (cid, id) SELECT @cid, @oid

    SET @disp = @disp + ', Created (' + @cid + ', ' + CAST(@oid AS NVARCHAR) + ')'
  END

  TRUNCATE TABLE #bulkcreatetable
  TRUNCATE TABLE #inst_attrs

  -- Finish up transaction
  IF @tcount = 0 
    COMMIT TRAN cl_createResource

  RETURN @oid
