PRINT 'Creating procedure cl_deleteLinksInBulk'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='cl_deleteLinksInBulk')
  DROP PROCEDURE cl_deleteLinksInBulk
GO

CREATE PROCEDURE cl_deleteLinksInBulk
AS
  -- input in #del_links
/*
  CREATE TABLE #del_links (
    type NVARCHAR(255),
    src_cid ClassID,
    src_id ObjID,
    dst_cid ClassID,
    dst_id ObjID
  )
*/

  RAISERROR('cl_deleteLinksInBulk: ', 0, 1)

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN cl_deleteLinksInBulk
  ELSE
    SAVE TRAN cl_deleteLinksInBulk

  -- Table to notify link changes
  DECLARE @cnt INT
  CREATE TABLE #linkChanges (
    type NVARCHAR(4),
    src_cid ClassID,
    src_id ObjID,
    dst_cid ClassID,
    dst_id ObjID,
    action_type sysname
  )

  /* Link Deletion in Bulk */
  UPDATE l SET l.deleted=1 FROM link l JOIN #del_links el 
    ON (l.link_type=el.type AND l.src_cid=el.src_cid AND l.src_id=el.src_id AND l.dst_cid=el.dst_cid AND l.dst_id=el.dst_id)
    WHERE l.deleted = 0
  SET @cnt = @@ROWCOUNT

  UPDATE c SET deleted=1 FROM containment c 
    JOIN #del_links el ON (c.src_cid=el.src_cid AND c.src_id=el.src_id AND c.dst_cid=el.dst_cid AND c.dst_id=el.dst_id)
    JOIN link_type lt ON (lt.link_type=el.type)
    JOIN link l ON (l.link_type=el.type AND l.src_cid=el.src_cid AND l.src_id=el.src_id AND l.dst_cid=el.dst_cid AND l.dst_id=el.dst_id)
    WHERE lt.flag_containment=1 AND l.deleted=0

  /* Link Notification */
  INSERT #linkChanges(type, src_cid, src_id, dst_cid, dst_id, action_type)
    SELECT type, src_cid, src_id, dst_cid, dst_id, 'LinkDeleted' FROM #del_links
  EXEC cl_notifyLinkChanges

  -- Finish up transaction
  IF @tcount = 0 
    COMMIT TRAN cl_deleteLinksInBulk

  RETURN @cnt
