PRINT 'Creating procedure cl_printLog'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='cl_printLog')
  DROP PROCEDURE cl_printLog
GO

CREATE PROCEDURE cl_printLog
@option NVARCHAR(64) = ''
AS
  RAISERROR('cl_printLog: @option=%s', 0, 1, @option)
/*
  Statistics of CL_IDCache are always traced irrespective of @option
  @option = ALL (traces all tables)
            ERROR_MSGS (traces only ERROR messages in CL_Status)
*/
  SET NOCOUNT ON
  SET @option = UPPER(@option)

  -- Get registration information of Instrumentations
  IF @option = 'ALL' BEGIN
    RAISERROR('***** Registration information of Instrumentations: *****', 0, 1)
    SELECT * FROM CL_Registration ORDER BY registrationId
  END

  -- Get transactions that are NOT in state 5
  IF @option = 'ALL' BEGIN
    RAISERROR('***** Transactions that are not processed (trState between 1 and 4): *****', 0, 1)
    SELECT * FROM CL_Stage WHERE trState BETWEEN 1 AND 4 ORDER BY trId
  END

  -- Get transactions that are in state 5
  IF @option = 'ALL' BEGIN
    RAISERROR('***** Processed transactions (trState=5): *****', 0, 1)
    SELECT * FROM CL_Stage WHERE trState=5 ORDER BY trId
  END

  -- Get transactions that are in state 6
  IF @option = 'ALL' BEGIN
    RAISERROR('***** Processed transactions that ended in deadlock etc (trState=6): *****', 0, 1)
    SELECT * FROM CL_Stage WHERE trState=6 ORDER BY trId
  END

  -- Get status data for transactions in state 5 (completed)
  RAISERROR('***** Status of transactions that are processed (trState=5): *****', 0, 1)
  IF @option = 'ERROR_MSGS'
    SELECT s.* FROM CL_Status s JOIN CL_Stage st ON (st.trId=s.trId) 
      WHERE st.trState=5 AND s.status='ERROR' ORDER BY s.time
  ELSE
    SELECT s.* FROM CL_Status s JOIN CL_Stage st ON (st.trId=s.trId) 
      WHERE st.trState=5 ORDER BY s.time

  -- Get status data for transactions in state 6 
  RAISERROR('***** Status of transactions that ended in deadlock etc (trState=6): *****', 0, 1)
  IF @option = 'ERROR_MSGS'
    SELECT s.* FROM CL_Status s JOIN CL_Stage st ON (st.trId=s.trId) 
      WHERE st.trState=6 AND s.status='ERROR' ORDER BY s.time
  ELSE
    SELECT s.* FROM CL_Status s JOIN CL_Stage st ON (st.trId=s.trId) 
      WHERE st.trState=6 ORDER BY s.time

  -- Get status data for transactions in state 4 (incomplete info)
  RAISERROR('***** Status of transactions that are being processed (trState=4): *****', 0, 1)
  IF @option = 'ERROR_MSGS'
    SELECT s.* FROM CL_Status s JOIN CL_Stage st ON (st.trId=s.trId) 
      WHERE st.trState=4 AND s.status='ERROR' ORDER BY s.time
  ELSE
    SELECT s.* FROM CL_Status s JOIN CL_Stage st ON (st.trId=s.trId) 
      WHERE st.trState=4 ORDER BY s.time

  -- Get data from CL_IDCache
  RAISERROR('***** Statistics from the CommonListener cache: *****', 0, 1)
  DECLARE @cnt INT
  SELECT @cnt=COUNT(*) FROM CL_IDCache WHERE keyword='CL_InstID' AND deleted=0
  RAISERROR('Total live objects in CL cache = %d', 0, 1, @cnt)
  SELECT @cnt=COUNT(*) FROM CL_IDCache WHERE keyword='CL_InstID' AND deleted=1
  RAISERROR('No of objects marked deleted in CL cache = %d', 0, 1, @cnt)

  SELECT cid AS ClassID, COUNT(oid) AS Object_Count FROM CL_IDCache 
    WHERE keyword='CL_InstID' AND deleted=0 GROUP BY cid 

  -- Get data from CL_Scheduled_SPCalls
  IF @option = 'ALL' BEGIN
    RAISERROR('***** Statistics from the CL_Scheduled_SPCalls: *****', 0, 1)
    SELECT @cnt=COUNT(*) FROM CL_Scheduled_SPCalls WHERE state=0
    RAISERROR('No of Scheduled SP calls pending = %d', 0, 1, @cnt)
    SELECT @cnt=COUNT(*) FROM CL_Scheduled_SPCalls WHERE state=1
    RAISERROR('No of Scheduled SP calls completed = %d', 0, 1, @cnt)
  END

  -- Get data from GEM_UpdateLOBView
  IF @option = 'ALL' BEGIN
    RAISERROR('***** Statistics from the GEM_UpdateLOBView: *****', 0, 1)
    SELECT @cnt=COUNT(*) FROM GEM_UpdateLOBView WHERE id>0
    RAISERROR('No of LOB updates when (cid, id) is known = %d', 0, 1, @cnt)
    SELECT @cnt=COUNT(*) FROM GEM_UpdateLOBView WHERE id=0
    RAISERROR('No of LOB updates when (cid) is known = %d', 0, 1, @cnt)
  END
