PRINT 'Creating procedure cl_processDelta'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='cl_processDelta')
  DROP PROCEDURE cl_processDelta
GO

CREATE PROCEDURE cl_processDelta
@cid ClassID,
@action TINYINT = 1,
@disp VARCHAR(8000) OUTPUT
AS
  /* All the attributes are in #attrs table
     @action = 1, 2, 3 (add, delete, update). Default is add        
  */
  RAISERROR('cl_processDelta: @cid=%s, @action=%d, @disp OUT', 0, 1, @cid, @action)

  CREATE TABLE #attrs_in (
    keyword NVARCHAR(64),
    value NVARCHAR(255),
    cmptype NVARCHAR(16)
  )
  CREATE TABLE #oids (
    oid ObjID
  )
  CREATE TABLE #inst_attrs(
    oid ObjID, 
    keyword nvarchar(64), 
    value nvarchar(255)
  )
  DECLARE @oid ObjID, @instrid NVARCHAR(255), @procstr NVARCHAR(128)

  -- Can only do delete action when cid is not-specified 
  -- [added to support NetView delta-deletes]
  IF @cid = '%' BEGIN
    IF @action = 2 BEGIN
      DECLARE @cnt INT
      SELECT @instrid=value FROM #attrs WHERE root='CommonListener' AND keyword='instrId'
      SELECT b.cid, b.oid INTO #delta_delete FROM #attrs a, CL_IDCache b 
        WHERE a.keyword='instid' AND a.root='inst' AND a.value=b.value AND 
	      b.keyword='CL_InstID' AND b.deleted=0 AND b.instrID=@instrid
      SET @cnt = @@ROWCOUNT

      IF @cnt = 0 
        SET @disp = @disp + ', Could not find a resource to delete'
      ELSE IF @cnt = 1 BEGIN
        SELECT @cid=cid, @oid=oid FROM #delta_delete
        SET @procstr = 'delete_' + @cid
        EXEC @procstr @oid
        SET @disp = @disp + ', deleted (' + @cid + ', ' + CAST(@oid AS NVARCHAR) + ')'
      END
      ELSE BEGIN
        -- Schedule deletes and mark then deleted in CL_IDCache    
        INSERT CL_Scheduled_SPCalls (sp_call)
          SELECT 'delete_' + cid + ' ' + CAST(oid AS NVARCHAR) FROM #delta_delete
        UPDATE c SET c.deleted=1 FROM CL_IDCache c JOIN #delta_delete d
          ON (c.cid=d.cid AND c.oid=d.oid)
        SELECT @disp=@disp + ', Scheduled deletes (' + cid + ', ' + CAST(oid AS NVARCHAR) + ')' FROM #delta_delete
      END
    END
    ELSE SET @disp = @disp + ', Invalid - as ClassID is % (match all ClassIDs) and action is not delete'		 

    RETURN
  END

  -- When valid CID is given (not %)
  SELECT @oid=b.oid, @instrid=b.instrID FROM #attrs a, CL_IDCache b 
    WHERE a.keyword='instid' AND a.root='inst' AND a.value=b.value AND 
	  b.cid=@cid AND b.keyword='CL_InstID' AND b.deleted=0

  IF @oid IS NULL BEGIN
    IF @action = 1 BEGIN
      EXEC @oid = cl_createResource @cid, @disp OUTPUT
      RETURN @oid
    END
    ELSE 
      SET @disp = @disp + ', Could not find a resource to update/delete'
  END
  ELSE BEGIN
    IF @action = 2 BEGIN
      IF EXISTS (SELECT * FROM #attrs WHERE root='CommonListener' AND keyword='instrId' AND value=@instrid) BEGIN
        SET @procstr = 'delete_' + @cid
        EXEC @procstr @oid
        SET @disp = @disp + ', deleted (' + @cid + ', ' + CAST(@oid AS NVARCHAR) + ')'
      END
      ELSE
        SET @disp = @disp + ', Cannot delete (' + @cid + ', ' + CAST(@oid AS NVARCHAR) + ') as its created by different Instrumentation'
    END
    ELSE BEGIN
      SET @procstr = 'cl_process' + @cid
      INSERT #inst_attrs SELECT @oid, keyword, value FROM #attrs WHERE root='inst' OR root='inst:attr1'

      IF @action = 1
        EXEC (@procstr + ' @action=''ADD_UPDATE''')
      ELSE 
        EXEC (@procstr)
      SET @disp = @disp + ', Updated (' + @cid + ', ' + CAST(@oid AS NVARCHAR) + ')'
    END    
  END

